@@ -27,6 +27,7 @@
 #include "VSplitter.h"
 #include "HSplitter.h"
 #include "BaseWindow.h"
+#include "MenuPage.h"
 
 namespace nux
 {
@@ -41,11 +42,18 @@ namespace nux
     ,   _max_size (AREA_MAX_WIDTH, AREA_MAX_HEIGHT)
   {
     _parent_area = NULL;
+    next_object_to_key_focus_area_ = NULL;
+    has_key_focus_ = false;
+
     _stretch_factor = 1;
     _layout_properties = NULL;
     _visible = true;
     _sensitive = true;
 
+    _on_geometry_changeg_reconfigure_parent_layout = true;
+    _accept_mouse_wheel_event = false;
+    _accept_keyboard_event = false;
+
     _2d_xform.Identity ();
     _3d_xform.Identity ();
     _3d_area = false;
@@ -70,71 +78,61 @@ namespace nux
 
   void Area::CheckMinSize()
   {
-    int w = _min_size.GetWidth();
+    int w = _min_size.width;
     w = Max<int>(AREA_MIN_WIDTH, w);
-    int h = _min_size.GetHeight();
+    int h = _min_size.height;
     h = Max<int>(AREA_MIN_HEIGHT, h);
 
-    _min_size.SetWidth(w);
-    _min_size.SetHeight(h);
+    _min_size = Size(w, h);
 
-    if (_min_size.GetWidth() > _max_size.GetWidth())
+    if (_min_size.width > _max_size.width)
     {
-      //temp = _max_size.GetWidth();
-      _max_size.SetWidth (_min_size.GetWidth());
-      //_min_size.SetWidth(temp);
+      _max_size.width = _min_size.width;
     }
 
-    if (_min_size.GetHeight() > _max_size.GetHeight())
+    if (_min_size.height > _max_size.height)
     {
-      //temp = _max_size.GetBaseHeight();
-      _max_size.SetHeight (_min_size.GetHeight());
-      //_min_size.SetHeight(temp);
+      _max_size.height = _min_size.height;
     }
 
-    if (_geometry.GetWidth() < _min_size.GetWidth() )
+    if (_geometry.width < _min_size.width)
     {
-      _geometry.SetWidth (_min_size.GetWidth() );
+      _geometry.width = _min_size.width;
     }
 
-    if (_geometry.GetHeight() < _min_size.GetHeight() )
+    if (_geometry.height < _min_size.height )
     {
-      _geometry.SetHeight (_min_size.GetHeight() );
+      _geometry.height = _min_size.height;
     }
   }
 
   void Area::CheckMaxSize()
   {
-    int w = _max_size.GetWidth();
+    int w = _max_size.width;
     w = Min<int>(AREA_MAX_WIDTH, w);
-    int h = _max_size.GetHeight();
+    int h = _max_size.height;
     h = Min<int>(AREA_MAX_HEIGHT, h);
 
-    _max_size.SetWidth(w);
-    _max_size.SetHeight(h);
+    _max_size = Size(w, h);
 
-    if (_min_size.GetWidth() > _max_size.GetWidth() )
+    if (_min_size.width > _max_size.width)
     {
-      //temp = _max_size.GetWidth();
-      _min_size.SetWidth (_max_size.GetWidth() );
-      //_min_size.SetWidth(temp);
+      _min_size.width = _max_size.width;
     }
 
-    if (_min_size.GetHeight() > _max_size.GetHeight() )
+    if (_min_size.height > _max_size.height)
     {
-      //temp = _max_size.GetBaseHeight();
-      _min_size.SetHeight (_max_size.GetHeight() );
-      //_min_size.SetHeight(temp);
+      _min_size.height = _max_size.height;
     }
 
-    if (_geometry.GetWidth() > _max_size.GetWidth() )
+    if (_geometry.width > _max_size.width)
     {
-      _geometry.SetWidth (_max_size.GetWidth() );
+      _geometry.width = _max_size.width;
     }
 
-    if (_geometry.GetHeight() > _max_size.GetHeight() )
+    if (_geometry.height > _max_size.height)
     {
-      _geometry.SetHeight (_max_size.GetHeight() );
+      _geometry.height = _max_size.height;
     }
   }
 
@@ -142,24 +140,22 @@ namespace nux
   {
     nuxAssert (w >= 0);
     nuxAssert (h >= 0);
-    _min_size.SetWidth (w);
-    _min_size.SetHeight (h);
+    _min_size = Size(w, h);
 
     CheckMinSize();
 
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::SetMaximumSize (int w, int h)
   {
     nuxAssert (w >= 0);
     nuxAssert (h >= 0);
-    _max_size.SetWidth (w);
-    _max_size.SetHeight (h);
+    _max_size = Size(w, h);
 
     CheckMaxSize();
 
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::SetMinMaxSize (int w, int h)
@@ -169,35 +165,35 @@ namespace nux
     SetMinimumSize (w, h);
     SetMaximumSize (w, h);
 
-    //InitiateResizeLayout();
+    //ReconfigureParentLayout();
   }
 
   void Area::ApplyMinWidth()
   {
-    _geometry.SetWidth (_min_size.GetWidth() );
+    _geometry.width = _min_size.width;
 
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::ApplyMinHeight()
   {
-    _geometry.SetHeight (_min_size.GetHeight() );
+    _geometry.height = _min_size.height;
 
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::ApplyMaxWidth()
   {
-    _geometry.SetWidth (_max_size.GetWidth() );
+    _geometry.width = _max_size.width;
 
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::ApplyMaxHeight()
   {
-    _geometry.SetHeight (_max_size.GetHeight() );
+    _geometry.height = _max_size.height;
 
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   Size Area::GetMinimumSize() const
@@ -213,53 +209,53 @@ namespace nux
   void Area::SetMinimumWidth (int w)
   {
     nuxAssert (w >= 0);
-    _min_size.SetWidth (w);
+    _min_size.width = w;
     CheckMinSize();
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::SetMaximumWidth (int w)
   {
     nuxAssert (w >= 0);
-    _max_size.SetWidth (w);
+    _max_size.width = w;
     CheckMaxSize();
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::SetMinimumHeight (int h)
   {
     nuxAssert (h >= 0);
-    _min_size.SetHeight (h);
+    _min_size.height = h;
     CheckMinSize();
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   void Area::SetMaximumHeight (int h)
   {
     nuxAssert (h >= 0);
-    _max_size.SetHeight (h);
+    _max_size.height = h;
     CheckMaxSize();
-    InitiateResizeLayout();
+    ReconfigureParentLayout();
   }
 
   int Area::GetMinimumWidth() const
   {
-    return _min_size.GetWidth();
+    return _min_size.width;
   }
 
   int Area::GetMaximumWidth() const
   {
-    return _max_size.GetWidth();
+    return _max_size.width;
   }
 
   int Area::GetMinimumHeight() const
   {
-    return _min_size.GetHeight();
+    return _min_size.height;
   }
 
   int Area::GetMaximumHeight() const
   {
-    return _max_size.GetHeight();
+    return _max_size.height;
   }
 
   unsigned int Area::GetStretchFactor()
@@ -317,28 +313,29 @@ namespace nux
 
   int Area::GetBaseWidth    () const
   {
-    return _geometry.GetWidth();
+    return _geometry.width;
   }
 
   int Area::GetBaseHeight   () const
   {
-    return _geometry.GetHeight();
+    return _geometry.height;
   }
 
-  void Area::SetGeometry (int x, int y, int w, int h)
+  void Area::SetGeometry(int x, int y, int w, int h)
   {
-    h = nux::Clamp<int> (h, _min_size.GetHeight (), _max_size.GetHeight ());
-    w = nux::Clamp<int> (w, _min_size.GetWidth (), _max_size.GetWidth ());
+    h = nux::Clamp<int> (h, _min_size.height, _max_size.height);
+    w = nux::Clamp<int> (w, _min_size.width, _max_size.width);
 
-    if (_geometry.x == x && _geometry.y == y && _geometry.width == w && _geometry.height == h)
+    nux::Geometry geometry(x, y, w, h);
+    if (_geometry == geometry)
       return;
 
-    GeometryChangePending ();
-    _geometry = nux::Geometry (x, y, w, h);
-    InitiateResizeLayout();
-    GeometryChanged ();
+    GeometryChangePending();
+    _geometry = geometry;
+    ReconfigureParentLayout();
+    GeometryChanged();
 
-    OnGeometryChanged.emit (this, _geometry);
+    OnGeometryChanged.emit(this, _geometry);
   }
 
   void Area::SetGeometry (const Geometry &geo)
@@ -346,14 +343,14 @@ namespace nux
     SetGeometry (geo.x, geo.y, geo.width, geo.height);
   }
 
-  Geometry Area::GetGeometry() const
+  Geometry const& Area::GetGeometry() const
   {
     return _geometry;
   }
 
-  void Area::SetBaseX    (int x)
+  void Area::SetBaseX(int x)
   {
-    SetGeometry (x, _geometry.y, _geometry.width, _geometry.height);
+    SetGeometry(x, _geometry.y, _geometry.width, _geometry.height);
   }
 
   void Area::SetBaseY    (int y)
@@ -384,7 +381,7 @@ namespace nux
   void Area::IncreaseSize (int x, int y)
   {
     _geometry.OffsetPosition (x, y);
-    OnResize.emit (_geometry.x, _geometry.y, _geometry.GetWidth(), _geometry.GetHeight() );
+    OnResize.emit (_geometry.x, _geometry.y, _geometry.width, _geometry.height );
   }
 
   long Area::ComputeChildLayout()
@@ -407,8 +404,21 @@ namespace nux
 
   }
 
-  void Area::InitiateResizeLayout (Area *child)
+  void Area::SetReconfigureParentLayoutOnGeometryChange(bool reconfigure_parent_layout)
   {
+    _on_geometry_changeg_reconfigure_parent_layout = reconfigure_parent_layout;
+  }
+  
+  bool Area::ReconfigureParentLayoutOnGeometryChange()
+  {
+    return _on_geometry_changeg_reconfigure_parent_layout;
+  }
+
+  void Area::ReconfigureParentLayout(Area *child)
+  {
+    if(_on_geometry_changeg_reconfigure_parent_layout == false)
+      return;
+
     if (GetWindowThread ()->IsComputingLayout() )
     {
       // there is no need to do the following while we are already computing the layout.
@@ -425,7 +435,7 @@ namespace nux
         this->Reference();
       }
 
-      View *ic = NUX_STATIC_CAST (View *, this);
+      View *ic = static_cast<View *>(this);
 
       if (ic->CanBreakLayout() )
       {
@@ -443,7 +453,7 @@ namespace nux
         }
       }
       else if (ic->_parent_area)
-        ic->_parent_area->InitiateResizeLayout (this);
+        ic->_parent_area->ReconfigureParentLayout (this);
       else
       {
         GetWindowThread ()->QueueObjectLayout (ic);
@@ -458,7 +468,7 @@ namespace nux
         this->Reference();
       }
 
-      Layout *layout = NUX_STATIC_CAST (Layout *, this);
+      Layout *layout = static_cast<Layout *>(this);
 
       if (layout->_parent_area)
       {
@@ -483,12 +493,12 @@ namespace nux
           else
           {
             // The parent object of an object of type View is a Layout object type.
-            layout->_parent_area->InitiateResizeLayout (this);
+            layout->_parent_area->ReconfigureParentLayout (this);
           }
         }
         else
         {
-          layout->_parent_area->InitiateResizeLayout (this);
+          layout->_parent_area->ReconfigureParentLayout (this);
         }
       }
       else
@@ -510,7 +520,7 @@ namespace nux
         }
 
         // The parent object of an object of type InputArea is a Layout object type.
-        this->_parent_area->InitiateResizeLayout (this);
+        this->_parent_area->ReconfigureParentLayout (this);
       }
     }
   }
@@ -661,57 +671,49 @@ namespace nux
     return _3d_area;
   }
 
-  static Geometry MatrixXFormGeometry (const Matrix4 &matrix, Geometry geo)
+  static void MatrixXFormGeometry (const Matrix4 &matrix, Geometry &geo)
   {
     Vector4 in (geo.x, geo.y, 0, 1);
     // This is mean only for translation matrices. It will not work with matrices containing rotations or scalings.
     Vector4 out = matrix * in;
-    Geometry new_geometry = Geometry (out.x, out.y, geo.width, geo.height);
-
-    return new_geometry;
+    geo.x = out.x;
+    geo.y = out.y;
   }
 
-  Geometry Area::InnerGetAbsoluteGeometry (const Geometry &geometry)
+  void Area::InnerGetAbsoluteGeometry (Geometry &geometry)
   {
-    Geometry new_geometry = geometry;
-
     if (this->Type ().IsDerivedFromType (BaseWindow::StaticObjectType) || (this == GetWindowThread ()->GetMainLayout ()))
     {
-      new_geometry.OffsetPosition (_geometry.x, _geometry.y);
-      return new_geometry;
+      geometry.OffsetPosition (_geometry.x, _geometry.y);
+      return;
     }
 
+    MatrixXFormGeometry (_2d_xform, geometry);
+
     Area *parent = GetParentObject ();
-    if (parent == 0)
-    {
-      //nuxDebugMsg (TEXT("[Area::InnerGetAbsoluteGeometry] Cannot reach the top level parent .This area may not be correctly parented"));
-      return MatrixXFormGeometry (_2d_xform, new_geometry);
-    }
-    else
-    {
-      return parent->InnerGetAbsoluteGeometry (MatrixXFormGeometry (_2d_xform, new_geometry));
-    }
+    if (parent)
+      parent->InnerGetAbsoluteGeometry (geometry);
   }
 
   Geometry Area::GetAbsoluteGeometry () const
   {
-    if (Type ().IsDerivedFromType (BaseWindow::StaticObjectType) || (this == GetWindowThread ()->GetMainLayout ()))
+    if (Type().IsDerivedFromType(BaseWindow::StaticObjectType) ||
+      Type().IsDerivedFromType(MenuPage::StaticObjectType) ||
+      (this == GetWindowThread()->GetMainLayout()))
     {
       // Do not apply the _2D_xform matrix  to a BaseWindow or the main layout
       return _geometry;
     }
     else
     {
+      nux::Geometry geo = _geometry;
+      MatrixXFormGeometry (_2d_xform, geo);
+
       Area *parent = GetParentObject ();
-      if (parent == 0)
-      {
-        //nuxDebugMsg (TEXT("[Area::GetAbsoluteGeometry] Cannot reach the top level parent .This area may not be correctly parented"));
-        return MatrixXFormGeometry (_2d_xform, _geometry);
-      }
-      else
-      {
-        return parent->InnerGetAbsoluteGeometry (MatrixXFormGeometry (_2d_xform, _geometry));
-      }
+      if (parent)
+        parent->InnerGetAbsoluteGeometry (geo);
+
+      return geo;
     }
   }
 
@@ -735,43 +737,34 @@ namespace nux
     return GetAbsoluteGeometry ().height;
   }
 
-  Geometry Area::InnerGetRootGeometry (const Geometry &geometry)
+  void Area::InnerGetRootGeometry (Geometry &geometry)
   {
     if (this->Type ().IsDerivedFromType (BaseWindow::StaticObjectType) || (this == GetWindowThread ()->GetMainLayout ()))
-    {
-      return geometry;
-    }
+      return;
+
+    MatrixXFormGeometry (_2d_xform, geometry);
 
     Area *parent = GetParentObject ();
-    if (parent == 0)
-    {
-      //nuxDebugMsg (TEXT("[Area::InnerGetRootGeometry] Cannot reach the top level parent .This area may not be correctly parented"));
-      return MatrixXFormGeometry (_2d_xform, geometry);
-    }
-    else
-    {
-      return parent->InnerGetRootGeometry (MatrixXFormGeometry (_2d_xform, geometry));
-    }
+    if (parent)
+      parent->InnerGetRootGeometry (geometry);
   }
 
   Geometry Area::GetRootGeometry () const
   {
-    if (Type ().IsDerivedFromType (BaseWindow::StaticObjectType) || (this == GetWindowThread ()->GetMainLayout ()))
+    nux::Geometry geo = _geometry;
+    MatrixXFormGeometry (_2d_xform, geo);
+
+    if (Type().IsDerivedFromType(BaseWindow::StaticObjectType) || (this == GetWindowThread()->GetMainLayout()))
     {
-      return MatrixXFormGeometry (_2d_xform, _geometry);
+      return geo;
     }
     else
     {
       Area *parent = GetParentObject ();
-      if (parent == 0)
-      {
-        //nuxDebugMsg (TEXT("[Area::GetRootGeometry] Cannot reach the top level parent .This area may not be correctly parented"));
-        return MatrixXFormGeometry (_2d_xform, _geometry);
-      }
-      else
-      {
-        return parent->InnerGetRootGeometry (MatrixXFormGeometry (_2d_xform, _geometry));
-      }
+      if (parent)
+        parent->InnerGetRootGeometry (geo);
+
+      return geo;
     }
   }
 
@@ -814,7 +807,7 @@ namespace nux
   {
     Area* area = GetToplevel ();
 
-    if (area->IsViewWindow ())
+    if (area && area->IsViewWindow ())
       return area;
 
     return NULL;
@@ -829,6 +822,17 @@ namespace nux
     return false;
   }
 
+  bool Area::IsChildOf(Area* parent)
+  {
+    if (this == parent)
+      return true;
+
+    if (!parent || !_parent_area)
+      return false;
+
+    return _parent_area->IsChildOf(parent);    
+  }
+
   /* handles our focusable code */
   bool Area::DoGetFocused ()
   {
@@ -861,4 +865,150 @@ namespace nux
     nux::GetWindowThread ()->QueueObjectLayout (this);
   }
   
+  void Area::SetAcceptKeyboardEvent(bool accept_keyboard_event)
+  {
+    _accept_keyboard_event = accept_keyboard_event;
+  }
+
+  bool Area::AcceptKeyboardEvent() const
+  {
+    return _accept_keyboard_event;
+  }
+
+  void Area::SetAcceptMouseWheelEvent(bool accept_mouse_wheel_event)
+  {
+    _accept_mouse_wheel_event = accept_mouse_wheel_event;
+  }
+
+  bool Area::AcceptMouseWheelEvent() const
+  {
+    return _accept_mouse_wheel_event;
+  }
+
+  bool Area::TestMousePointerInclusion(const Point& mouse_position, NuxEventType event_type)
+  {
+    bool mouse_pointer_inside_area = false;
+
+    if (Type().IsDerivedFromType(MenuPage::StaticObjectType))
+    {
+      // A MenuPage geometry is already in absolute coordinates.
+      mouse_pointer_inside_area = _geometry.IsInside(mouse_position);
+    }
+    else
+    {
+      mouse_pointer_inside_area = GetAbsoluteGeometry().IsInside(mouse_position);
+    }
+
+    if ((event_type == NUX_MOUSE_WHEEL) && mouse_pointer_inside_area)
+    {
+      if (_accept_mouse_wheel_event == false)
+        return NULL;
+    }
+
+    return mouse_pointer_inside_area;
+  }
+
+  bool Area::TestMousePointerInclusionFilterMouseWheel(const Point& mouse_position, NuxEventType event_type)
+  {
+    bool mouse_pointer_inside_area = false;
+
+    if (Type().IsDerivedFromType(MenuPage::StaticObjectType))
+    {
+      // A MenuPage geometry is already in absolute coordinates.
+      mouse_pointer_inside_area = _geometry.IsInside(mouse_position);
+    }
+    else
+    {
+      mouse_pointer_inside_area = GetAbsoluteGeometry().IsInside(mouse_position);
+    }
+
+    return mouse_pointer_inside_area;
+  }
+
+  Area* Area::FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type)
+  {
+    return NULL;
+  }
+  
+  Area* Area::FindKeyFocusArea(unsigned int key_symbol,
+   unsigned long x11_key_code,
+   unsigned long special_keys_state)
+  {
+    if (has_key_focus_)
+    {
+      return this;
+    }
+    else if (next_object_to_key_focus_area_)
+    {
+      return next_object_to_key_focus_area_->FindKeyFocusArea(key_symbol, x11_key_code, special_keys_state);
+    }
+    return NULL;
+  }
+
+  void Area::SetPathToKeyFocusArea()
+  {
+    has_key_focus_ = true;
+    next_object_to_key_focus_area_ = NULL;
+
+    Area* child = this;
+    Area* parent = GetParentObject();
+
+    while (parent)
+    {
+      parent->next_object_to_key_focus_area_ = child;
+      parent->next_object_to_key_focus_area_->Reference();
+      parent->has_key_focus_ = false;
+      child = parent;
+      parent = parent->GetParentObject();
+    }
+  }
+
+  void Area::ResetDownwardPathToKeyFocusArea()
+  {
+    has_key_focus_ = false;
+    if (next_object_to_key_focus_area_)
+    {
+      next_object_to_key_focus_area_->ResetDownwardPathToKeyFocusArea();
+    }
+    if(next_object_to_key_focus_area_)
+      next_object_to_key_focus_area_->UnReference();
+
+    next_object_to_key_focus_area_ = NULL;
+  }
+
+  void Area::ResetUpwardPathToKeyFocusArea()
+  {
+    has_key_focus_ = false;
+    if (_parent_area)
+    {
+      _parent_area->ResetUpwardPathToKeyFocusArea();
+    }
+    if(next_object_to_key_focus_area_)
+      next_object_to_key_focus_area_->UnReference();
+
+    next_object_to_key_focus_area_ = NULL;
+  }
+
+  bool Area::InspectKeyEvent(unsigned int eventType,
+    unsigned int keysym,
+    const char* character)
+  {
+    return false;
+  }
+
+  bool Area::AcceptKeyNavFocus()
+  {
+    return true;
+  }
+
+  Area* Area::KeyNavIteration(KeyNavDirection direction)
+  {
+    return NULL;
+  }
+
+  bool Area::HasKeyFocus() const
+  {
+    return has_key_focus_;
+  }
 }
+
