@@ -54,31 +54,30 @@ namespace nux
     m_control_knot.push_back (Knot2 (0.7f, 0.9f) );
     m_control_knot.push_back (Knot2 (1.0f, 1.0f) );
 
-    OnMouseDown.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvMouseDown) );
-    OnMouseUp.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvMouseUp) );
-    OnMouseDrag.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvMouseDrag) );
+    mouse_down.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvMouseDown) );
+    mouse_up.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvMouseUp) );
+    mouse_drag.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvMouseDrag) );
 
-    OnKeyEvent.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvKeyEvent) );
+    key_down.connect (sigc::mem_fun (this, &BezierCurveControl2::RecvKeyEvent) );
 
     NTextureData image;
     MakeCheckBoardImage (image.GetSurface (0), 64, 64, Color (0xff323232), Color (0xff535353), 8, 8);
-    BaseTexture* m_CheckboardPattern = GetGpuDevice()->CreateSystemCapableTexture ();
+    BaseTexture* m_CheckboardPattern = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture ();
     m_CheckboardPattern->Update (&image);
 
     TexCoordXForm texxform;
     texxform.SetTexCoordType (TexCoordXForm::OFFSET_COORD);
     texxform.SetWrap (TEXWRAP_REPEAT, TEXWRAP_REPEAT);
-    m_Background = new TextureLayer (m_CheckboardPattern->GetDeviceTexture(), texxform, Colors::White);
+    m_Background = new TextureLayer (m_CheckboardPattern->GetDeviceTexture(), texxform, color::White);
 
     m_CheckboardPattern->UnReference ();
   }
 
   BezierCurveControl2::~BezierCurveControl2()
   {
-    NUX_SAFE_DELETE (m_Background);
+    delete m_Background;
   }
 
-
   long BezierCurveControl2::ProcessEvent (IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
   {
     long ret = TraverseInfo;
@@ -237,8 +236,8 @@ namespace nux
         }
       }
 
-      delete xcon;
-      delete ycon;
+      delete[] xcon;
+      delete[] ycon;
     }
 
     GetPainter().Paint2DQuadWireframe (GfxContext, base, Color (0xFF000000) );
@@ -582,20 +581,20 @@ namespace nux
   {
     m_minX = minX;
     m_maxX = maxX;
-    NeedRedraw();
+    QueueDraw();
   }
 
   void BezierCurveControl2::SetYAxisBounds (float minY, float maxY)
   {
     m_minY = minY;
     m_maxY = maxY;
-    NeedRedraw();
+    QueueDraw();
   }
 
   void BezierCurveControl2::SetFunctionCallback (FunctionCallback f)
   {
     m_FunctionCallback = f;
-    NeedRedraw();
+    QueueDraw();
   }
 
   float BezierCurveControl2::EvalFunction (float x)
@@ -608,7 +607,7 @@ namespace nux
 
   void BezierCurveControl2::UpdateGraph()
   {
-    NeedRedraw();
+    QueueDraw();
   }
 
 
@@ -624,7 +623,7 @@ namespace nux
 
   void BezierCurveControl2::RecvMouseUp (int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
-    NeedRedraw();
+    QueueDraw();
   }
 
   void BezierCurveControl2::RecvMouseDown (int x, int y, unsigned long button_flags, unsigned long key_flags)
@@ -682,7 +681,7 @@ namespace nux
       }
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   int ClosestCubicBezierEndPoint (int i)
@@ -706,7 +705,6 @@ namespace nux
   }
 
   void BezierCurveControl2::RecvKeyEvent (
-    GraphicsEngine  &GfxContext , /*Graphics Context for text operation*/
     unsigned long   eventType  , /*event type*/
     unsigned long   keysym     , /*event keysym*/
     unsigned long   state      , /*event state*/
@@ -993,7 +991,7 @@ namespace nux
       }
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void BezierCurveControl2::ProcessPanning (int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
@@ -1014,7 +1012,7 @@ namespace nux
         m_minY += yp;
         m_maxY += yp;
 
-        NeedRedraw();
+        QueueDraw();
         return;
       }
     }
@@ -1022,12 +1020,9 @@ namespace nux
 
   void BezierCurveControl2::ProcessZooming (int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
   {
-    float xp, yp;
-    int W = GetBaseWidth() - 2 * GRAPH_MARGIN;
     int H = GetBaseHeight() - 2 * GRAPH_MARGIN;
 
-    xp = /*m_minX +*/ dx * (m_maxX - m_minX) / W;
-    yp = /*m_minY +*/ dy * (m_maxY - m_minY) / H;
+    float yp = dy * (m_maxY - m_minY) / H;
 
     if ( (m_minX + yp >= m_maxX - yp) || (m_minY + yp >= m_maxY - yp) )
     {
@@ -1043,7 +1038,7 @@ namespace nux
         m_minY += yp;
         m_maxY -= yp;
 
-        NeedRedraw();
+        QueueDraw();
         return;
       }
     }
