@@ -68,79 +68,13 @@ namespace nux
     std::vector< MySplitter* >::iterator it2;
     for (it2 = m_SplitterObject.begin(); it2 != m_SplitterObject.end(); it2++)
     {
-      (*it2)->UnReference();
+      (*it2)->UnParentObject();
     }
     m_SplitterObject.clear();
 
     m_SplitConfig.clear();
   }
 
-  long HSplitter::ProcessEvent (IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
-  {
-    long ret = TraverseInfo;
-    long ProcEvInfo = ProcessEventInfo;
-
-    if (ievent.e_event == NUX_MOUSE_PRESSED)
-    {
-      if (!_geometry.IsPointInside (ievent.e_x, ievent.e_y) )
-      {
-        ProcEvInfo |= eDoNotProcess;
-        //return TraverseInfo;
-      }
-    }
-
-    //if(traverse)
-    {
-      std::vector<MySplitter *>::iterator it_splitter;
-
-      for (it_splitter = m_SplitterObject.begin(); it_splitter != m_SplitterObject.end(); it_splitter++)
-      {
-        ret = (*it_splitter)->OnEvent (ievent, ret, ProcEvInfo);
-      }
-
-      std::vector<Area *>::iterator it;
-
-      for (it = m_InterfaceObject.begin(),
-           it_splitter = m_SplitterObject.begin();
-           it != m_InterfaceObject.end();
-           it++, it_splitter++)
-      {
-        Geometry clip_geo;
-        clip_geo.SetX ( ( (*it)->GetGeometry().x) );
-        clip_geo.SetY ( ( (*it)->GetGeometry().y) );
-        clip_geo.SetWidth ( (*it)->GetGeometry().GetWidth() );
-        clip_geo.SetHeight ( (*it_splitter)->GetGeometry().y - (*it)->GetGeometry().y);
-
-        long DoNotProcess = ProcEvInfo;
-
-        if (clip_geo.IsPointInside (ievent.e_x, ievent.e_y) == false)
-        {
-          DoNotProcess |= eDoNotProcess;
-        }
-
-        if ( (*it)->Type().IsDerivedFromType (View::StaticObjectType) )
-        {
-          View *ic = NUX_STATIC_CAST (View *, (*it) );
-          ret = ic->BaseProcessEvent (ievent, ret, ProcEvInfo | DoNotProcess);
-        }
-        else if ( (*it)->Type().IsObjectType (InputArea::StaticObjectType) )
-        {
-          InputArea *base_area = NUX_STATIC_CAST (InputArea *, (*it) );
-          ret = base_area->OnEvent (ievent, ret, ProcEvInfo | DoNotProcess);
-        }
-        else if ( (*it)->Type().IsDerivedFromType (Layout::StaticObjectType) )
-        {
-          Layout *layout = NUX_STATIC_CAST (Layout *, (*it) );
-          ret = layout->ProcessEvent (ievent, ret, ProcEvInfo | DoNotProcess);
-        }
-      }
-    }
-
-    ret = PostProcessEvent2 (ievent, ret, ProcEvInfo);
-
-    return ret;
-  }
-
   void HSplitter::Draw (GraphicsEngine &GfxContext, bool force_draw)
   {
     GfxContext.PushClippingRectangle (GetGeometry() );
@@ -206,22 +140,22 @@ namespace nux
       {
         if ( (*it)->Type().IsDerivedFromType (View::StaticObjectType) )
         {
-          View *ic = NUX_STATIC_CAST (View *, (*it) );
+          View *ic = static_cast<View *>(*it);
           ic->ProcessDraw (GfxContext, true);
         }
         else if ( (*it)->Type().IsObjectType (InputArea::StaticObjectType) )
         {
-          InputArea *base_area = NUX_STATIC_CAST (InputArea *, (*it) );
+          InputArea *base_area = static_cast<InputArea *>(*it);
           base_area->OnDraw (GfxContext, true);
         }
         else if ( (*it)->Type().IsObjectType (HLayout::StaticObjectType) )
         {
-          HLayout *layout = NUX_STATIC_CAST (HLayout *, (*it) );
+          HLayout *layout = static_cast<HLayout *>(*it);
           layout->ProcessDraw (GfxContext, true);
         }
         else if ( (*it)->Type().IsObjectType (VLayout::StaticObjectType) )
         {
-          VLayout *layout = NUX_STATIC_CAST (VLayout *, (*it) );
+          VLayout *layout = static_cast<VLayout *>(*it);
           layout->ProcessDraw (GfxContext, true);
         }
       }
@@ -229,22 +163,22 @@ namespace nux
       {
         if ( (*it)->Type().IsDerivedFromType (View::StaticObjectType) )
         {
-          View *ic = NUX_STATIC_CAST (View *, (*it) );
+          View *ic = static_cast<View *>(*it);
           ic->ProcessDraw (GfxContext, false);
         }
         else if ( (*it)->Type().IsObjectType (InputArea::StaticObjectType) )
         {
-          InputArea *base_area = NUX_STATIC_CAST (InputArea *, (*it) );
+          InputArea *base_area = static_cast<InputArea *>(*it);
           base_area->OnDraw (GfxContext, false);
         }
         else if ( (*it)->Type().IsObjectType (HLayout::StaticObjectType) )
         {
-          HLayout *layout = NUX_STATIC_CAST (HLayout *, (*it) );
+          HLayout *layout = static_cast<HLayout *>(*it);
           layout->ProcessDraw (GfxContext, false);
         }
         else if ( (*it)->Type().IsObjectType (VLayout::StaticObjectType) )
         {
-          VLayout *layout = NUX_STATIC_CAST (VLayout *, (*it) );
+          VLayout *layout = static_cast<VLayout *>(*it);
           layout->ProcessDraw (GfxContext, false);
         }
       }
@@ -310,11 +244,13 @@ namespace nux
     if (ic)
     {
       MySplitter *splitter = new MySplitter;
-      splitter->SinkReference();
+      splitter->SetParentObject(this);
+      //splitter->SinkReference();
+
       t_u32 no = (t_u32) m_InterfaceObject.size();
-      splitter->OnMouseDown.connect (sigc::bind ( sigc::mem_fun (this, &HSplitter::OnSplitterMouseDown), no) );
-      splitter->OnMouseDrag.connect (sigc::bind ( sigc::mem_fun (this, &HSplitter::OnSplitterMouseDrag), no) );
-      splitter->OnMouseUp.connect (sigc::bind ( sigc::mem_fun (this, &HSplitter::OnSplitterMouseUp), no) );
+      splitter->mouse_down.connect (sigc::bind ( sigc::mem_fun (this, &HSplitter::OnSplitterMouseDown), no) );
+      splitter->mouse_drag.connect (sigc::bind ( sigc::mem_fun (this, &HSplitter::OnSplitterMouseDrag), no) );
+      splitter->mouse_up.connect (sigc::bind ( sigc::mem_fun (this, &HSplitter::OnSplitterMouseUp), no) );
 
       ic->SetParentObject (this);
       m_InterfaceObject.push_back (ic);
@@ -417,7 +353,7 @@ namespace nux
 
       if (m_InterfaceObject[i]->Type().IsDerivedFromType (View::StaticObjectType) )
       {
-        View *ic = NUX_STATIC_CAST (View *, m_InterfaceObject[i]);
+        View *ic = static_cast<View *>(m_InterfaceObject[i]);
         ic->SetGeometry (Geometry (x, accheight, w, splitter_geo.y - accheight) );
         // if we are already computing the layout from the main window down, we need to call
         // ComputeElementLayout to force the computing of this element layout.
@@ -425,12 +361,12 @@ namespace nux
       }
       else if (m_InterfaceObject[i]->Type().IsObjectType (InputArea::StaticObjectType) )
       {
-        InputArea *base_area = NUX_STATIC_CAST (InputArea *, m_InterfaceObject[i]);
+        InputArea *base_area = static_cast<InputArea *>(m_InterfaceObject[i]);
         base_area->SetGeometry (Geometry (x, accheight, w, splitter_geo.y - accheight) );
       }
       else if (m_InterfaceObject[i]->Type().IsDerivedFromType (Layout::StaticObjectType) )
       {
-        Layout *layout = NUX_STATIC_CAST (Layout *, m_InterfaceObject[i]);
+        Layout *layout = static_cast<Layout *>(m_InterfaceObject[i]);
         layout->SetGeometry (Geometry (x, accheight, w, splitter_geo.y - accheight) );
         // if we are already computing the layout from the main window down, we need to call
         // ComputeElementLayout to force the computing of this element layout.
@@ -501,7 +437,7 @@ namespace nux
 
   void HSplitter::OnSplitterMouseDown (t_s32 x, t_s32 y, unsigned long button_flags, unsigned long key_flags, t_s32 header_pos)
   {
-    m_point.Set (x, y);
+    m_point = Point(x, y);
 
     m_focus_splitter_index = header_pos;
     GetWindowCompositor().SetWidgetDrawingOverlay (this, GetWindowCompositor ().GetProcessingTopView () );
@@ -543,7 +479,6 @@ namespace nux
 
   void HSplitter::OnSplitterMouseDrag (t_s32 x, t_s32 y, t_s32 dx, t_s32 dy, unsigned long button_flags, unsigned long key_flags, t_s32 header_pos)
   {
-    bool recompute = false;
     Geometry geo = m_SplitterObject[header_pos]->GetGeometry();
     t_s32 num_element = (t_s32) m_SplitterObject.size();
 
@@ -553,7 +488,6 @@ namespace nux
       return;
     }
 
-    recompute = true;
     mvt_dx = 0;
     mvt_dy = (y - m_point.y);
 
@@ -615,7 +549,7 @@ namespace nux
     }
 
     ComputeChildLayout();
-    NeedRedraw();
+    QueueDraw();
   }
 
 // HSplitter need to re implement DoneRedraw because it does not
@@ -624,19 +558,162 @@ namespace nux
   {
     std::vector<Area *>::iterator it;
 
-    for (it = m_InterfaceObject.begin(); it != m_InterfaceObject.end(); it++)
+    for(it = m_InterfaceObject.begin(); it != m_InterfaceObject.end(); it++)
     {
       //(*it)->DoneRedraw();
-      if ( (*it)->Type().IsDerivedFromType (View::StaticObjectType) )
+      if((*it)->Type().IsDerivedFromType (View::StaticObjectType))
       {
-        View *ic = NUX_STATIC_CAST (View *, (*it) );
+        View *ic = static_cast<View *>(*it);
         ic->DoneRedraw();
       }
-      else if ( (*it)->Type().IsObjectType (InputArea::StaticObjectType) )
+
+      else if((*it)->Type().IsObjectType (InputArea::StaticObjectType))
       {
         //InputArea* base_area = NUX_STATIC_CAST(InputArea*, (*it));
       }
     }
   }
 
+  Area* HSplitter::FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type)
+  {
+    bool mouse_inside = TestMousePointerInclusionFilterMouseWheel(mouse_position, event_type);
+
+    if(mouse_inside == false)
+      return NULL;
+
+    std::vector<MySplitter*>::iterator splitter_it;
+    for (splitter_it = m_SplitterObject.begin(); splitter_it != m_SplitterObject.end(); splitter_it++)
+    {
+      Area* found_area = (*splitter_it)->FindAreaUnderMouse(mouse_position, event_type);
+      if(found_area)
+        return found_area;
+    }
+
+    std::vector<Area *>::iterator it;
+    for(it = m_InterfaceObject.begin(); it != m_InterfaceObject.end(); it++)
+    {
+      Area* found_area = (*it)->FindAreaUnderMouse(mouse_position, event_type);
+
+      if(found_area)
+        return found_area;
+    }
+
+    if((event_type == NUX_MOUSE_WHEEL) && (!AcceptMouseWheelEvent()))
+      return NULL;
+    return this;
+  }
+
+  bool HSplitter::AcceptKeyNavFocus()
+  {
+    return false;
+  }
+
+  Area* HSplitter::KeyNavIteration(KeyNavDirection direction)
+  {
+    if (m_InterfaceObject.size() == 0)
+      return NULL;
+
+    if (next_object_to_key_focus_area_)
+    {
+      if ((direction == KEY_NAV_LEFT) || (direction == KEY_NAV_RIGHT))
+      {
+        // Don't know what to do with this
+        return NULL;
+      }
+      std::vector<Area*>::iterator it;
+      std::vector<Area*>::iterator it_next;
+      it = std::find (m_InterfaceObject.begin(), m_InterfaceObject.end(), next_object_to_key_focus_area_);
+
+      if (it == m_InterfaceObject.end())
+      {
+        // Should never happen
+        nuxAssert (0);
+        return NULL;
+      }
+
+      it_next = it;
+      ++it_next;
+
+      if ((direction == KEY_NAV_UP) && (it == m_InterfaceObject.begin()))
+      {
+        // can't go further
+        return NULL;
+      }
+
+      if ((direction == KEY_NAV_DOWN) && (it_next == m_InterfaceObject.end()))
+      {
+        // can't go further
+        return NULL;
+      }
+
+      if ((direction == KEY_NAV_UP))
+      {
+        --it;
+        Area* key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          if (it == m_InterfaceObject.begin())
+            break;
+
+          --it;
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+
+        return key_nav_focus;
+      }
+
+      if ((direction == KEY_NAV_DOWN))
+      {
+        ++it;
+        Area* key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          ++it;
+          if (it == m_InterfaceObject.end())
+            break;
+
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+
+        return key_nav_focus;
+      }
+    }
+    else
+    {
+      Area* key_nav_focus = NULL;
+      if (direction == KEY_NAV_UP)
+      {
+        std::vector<Area*>::reverse_iterator it = m_InterfaceObject.rbegin();
+        key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          ++it;
+          if (it == m_InterfaceObject.rend())
+            break;
+
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+      }
+      else
+      {
+        std::vector<Area*>::iterator it = m_InterfaceObject.begin();
+        key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          ++it;
+          if (it == m_InterfaceObject.end())
+            break;
+
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+      }
+      return key_nav_focus;
+    }
+
+    return NULL;
+  }
 }
