@@ -217,14 +217,14 @@ namespace nux
 
   void MenuSeparator::Draw (GraphicsEngine &GfxContext, bool force_draw)
   {
-    Geometry base = GetGeometry ();
-    int y0 = base.y + base.GetHeight () / 2;
-    GetPainter ().Draw2DLine (GfxContext, base.x, y0, base.x + base.GetWidth(), y0, Color (0xFF222222));
-    GetPainter ().Draw2DLine (GfxContext, base.x, y0 + 1, base.x + base.GetWidth(), y0 + 1, Color (0xFFAAAAAA) );
+    Geometry base = GetGeometry();
+    int y0 = base.y + base.GetHeight() / 2;
+    GetPainter().Draw2DLine (GfxContext, base.x, y0, base.x + base.GetWidth(), y0, Color(0xFF222222));
+    GetPainter().Draw2DLine (GfxContext, base.x, y0 + 1, base.x + base.GetWidth(), y0 + 1, Color(0xFFAAAAAA));
   }
 
-  MenuPage::MenuPage (const TCHAR *title, NUX_FILE_LINE_DECL)
-    :   View (NUX_FILE_LINE_PARAM)
+  MenuPage::MenuPage(const TCHAR *title, NUX_FILE_LINE_DECL)
+  : View (NUX_FILE_LINE_PARAM)
   {
     m_Parent = 0;
     m_item_width = MENU_ITEM_MIN_WIDTH;
@@ -233,20 +233,22 @@ namespace nux
     m_MenuWindow = 0;
     m_Name = title;
     m_IsTopOfMenuChain = false;
-    _font_name = g_strdup ("Ubuntu 12");
+    _font_name = g_strdup("Ubuntu 12");
+
+    on_closure_continue_with_event_ = false;
 
     // Set Original State
 
     // Set Signals
-    OnMouseMove.connect (sigc::mem_fun (this, &MenuPage::EmitMouseMove));
-    OnMouseDrag.connect (sigc::mem_fun (this, &MenuPage::EmitMouseDrag));
-    OnMouseDown.connect (sigc::mem_fun (this, &MenuPage::EmitMouseDown));
-    OnMouseUp.connect (sigc::mem_fun (this, &MenuPage::EmitMouseUp));
-    OnMouseLeave.connect (sigc::mem_fun (this, &MenuPage::RecvMouseLeave));
-    OnMouseDownOutsideArea.connect (sigc::mem_fun (this, &MenuPage::Terminate));
+    mouse_move.connect(sigc::mem_fun(this, &MenuPage::EmitMouseMove));
+    mouse_drag.connect(sigc::mem_fun(this, &MenuPage::EmitMouseDrag));
+    mouse_down.connect(sigc::mem_fun(this, &MenuPage::EmitMouseDown));
+    mouse_up.connect(sigc::mem_fun(this, &MenuPage::EmitMouseUp));
+    mouse_leave.connect(sigc::mem_fun(this, &MenuPage::RecvMouseLeave));
+    mouse_down_outside_pointer_grab_area.connect(sigc::mem_fun (this, &MenuPage::Terminate));
 
     // Set Geometry
-    SetGeometry (Geometry (0, 0, DEFAULT_WIDGET_WIDTH, DEFAULT_WIDGET_HEIGHT));
+    SetGeometry(Geometry (0, 0, DEFAULT_WIDGET_WIDTH, DEFAULT_WIDGET_HEIGHT));
 
     // Set layout
 
@@ -256,12 +258,12 @@ namespace nux
     m_NextMouseUpMeanStop = false;
     m_SubMenuAction = 0;
 
-    vlayout = new VLayout (NUX_TRACKER_LOCATION);
+    _vlayout = new VLayout (NUX_TRACKER_LOCATION);
     // No Need to set a composition layout.
     // The MenuPage is floating above everything else.
-    SetLayout (vlayout);
+    SetLayout(_vlayout);
 
-    SetTextColor (Colors::Black);
+    SetTextColor (color::Black);
   }
 
   MenuPage::~MenuPage ()
@@ -304,7 +306,7 @@ namespace nux
     {
       if (mod_event.e_event == NUX_MOUSE_RELEASED)
       {
-        Geometry geo = GetThreadGLWindow()->GetWindowGeometry();
+        Geometry geo = GetGraphicsDisplay()->GetWindowGeometry();
         geo.SetX (0);
         geo.SetY (0);
 
@@ -320,7 +322,7 @@ namespace nux
       }
       else if (mod_event.e_event == NUX_MOUSE_PRESSED)
       {
-        Geometry geo = GetThreadGLWindow()->GetWindowGeometry();
+        Geometry geo = GetGraphicsDisplay()->GetWindowGeometry();
         geo.SetX (0);
         geo.SetY (0);
 
@@ -345,7 +347,7 @@ namespace nux
       else
       {
         if (mod_event.e_event == NUX_MOUSE_MOVE)
-          NeedRedraw ();
+          QueueDraw ();
 
         ret = PostProcessEvent2 (mod_event, ret, ProcessEventInfo);
       }
@@ -359,6 +361,18 @@ namespace nux
     return ret;
   };
 
+  Area* MenuPage::FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type)
+  {
+    bool mouse_inside = TestMousePointerInclusionFilterMouseWheel(mouse_position, event_type);
+
+    if(mouse_inside == false)
+      return NULL;
+
+    if((event_type == NUX_MOUSE_WHEEL) && (!AcceptMouseWheelEvent()))
+      return NULL;
+    return this;
+  }
+
   void MenuPage::Draw (GraphicsEngine &GfxContext, bool force_draw)
   {
     if (m_IsActive)
@@ -493,7 +507,7 @@ namespace nux
     }
 
     m_numItem = (int) m_MenuItemVector.size();
-    vlayout->AddView (pMenuItem, 0, eLeft, eFix);
+    _vlayout->AddView (pMenuItem, 0, eLeft, eFix);
     ComputeChildLayout();
 
     return pMenuItem->GetActionItem();
@@ -567,7 +581,7 @@ namespace nux
 //    }
 //
 //    m_numItem = (int)m_MenuItemVector.size();
-//    vlayout->AddView(pMenuItem, 0, eLeft, eFix);
+//    _vlayout->AddView(pMenuItem, 0, eLeft, eFix);
 //    ComputeChildLayout();
 //}
 
@@ -639,7 +653,7 @@ namespace nux
     }
 
     m_numItem = (int) m_MenuItemVector.size();
-    vlayout->AddView (pMenuItem, 0, eLeft, eFix);
+    _vlayout->AddView (pMenuItem, 0, eLeft, eFix);
     ComputeChildLayout();
 
     return pMenuItem->GetChildMenu();
@@ -712,7 +726,7 @@ namespace nux
     }
 
     m_numItem = (int) m_MenuItemVector.size();
-    vlayout->AddView (pMenuItem, 0, eLeft, eFix);
+    _vlayout->AddView (pMenuItem, 0, eLeft, eFix);
     ComputeChildLayout();
 
     return pMenuItem->GetActionItem();
@@ -740,7 +754,7 @@ namespace nux
                                      + MENU_ITEM_TEXT_TO_BORDER_MARGIN, 4);
     }
 
-    vlayout->AddView (pMenuSeparator, 0, eLeft, eFix);
+    _vlayout->AddView (pMenuSeparator, 0, eLeft, eFix);
     ComputeChildLayout();
   }
 
@@ -753,7 +767,7 @@ namespace nux
     m_MenuSeparatorVector.clear();
     m_MenuItemVector.clear();
     m_numItem = 0;
-    vlayout->Clear();
+    _vlayout->Clear();
     ComputeChildLayout();
    
     //FIXME - Hack to fix a bug with the menu height not being reset after removing items
@@ -781,20 +795,25 @@ namespace nux
         if ((y >= py) && (y < py + height))
         {
           m_HighlightedItem = i;
+          QueueDraw();
           break;
         }
       }
     }
     else
     {
-      m_HighlightedItem = -1;
+      if(m_HighlightedItem != -1)
+      {
+        m_HighlightedItem = -1;
+        QueueDraw();
+      }
     }
 
     if (m_HighlightedItem >= 0)
     {
       MenuItem *selected_action = m_MenuItemVector[m_HighlightedItem];
 
-      if ( (selected_action->GetChildMenu() != 0) && selected_action->GetActionItem()->isEnabled() )
+      if((selected_action->GetChildMenu() != 0) && selected_action->GetActionItem()->isEnabled())
       {
         // This MenuItem has a sub-MenuPage. Start it.
         Geometry geo = selected_action->GetGeometry();
@@ -802,7 +821,7 @@ namespace nux
         selected_action->GetChildMenu()->StartMenu (geo.x + geo.GetWidth() - 5, geo.y, 0, 0);
 
         // The current SubMenu is not the same as the new one...
-        if (m_SubMenuAction != selected_action)
+        if(m_SubMenuAction != selected_action)
         {
           // If m_SubMenuAction is not null then stop the sub menu
           StopActionSubMenu();
@@ -968,6 +987,8 @@ namespace nux
         m_HighlightedItem = -1;
       }
     }
+
+    QueueDraw();
   }
 
   void MenuPage::StartMenu (int MenuXPosition, int MenuYPosition, int x, int y, bool OverrideCurrentMenuChain)
@@ -1194,5 +1215,23 @@ namespace nux
     return -1;
   }
 
+  Geometry MenuPage::GetAbsoluteGeometry () const
+  {
+    return GetGeometry();  
+  }
 
+  Geometry MenuPage::GetRootGeometry () const
+  {
+    return GetGeometry();
+  }
+
+  void MenuPage::SetOnClosureContinueEventCycle(bool on_closure_continue_with_event)
+  {
+    on_closure_continue_with_event_ = on_closure_continue_with_event;
+  }
+
+  bool MenuPage::OnClosureContinueEventCycle() const
+  {
+    return on_closure_continue_with_event_;
+  }
 }
