@@ -172,6 +172,7 @@ namespace nux
 
   BasePainter::~BasePainter()
   {
+    EmptyBackgroundStack();
   }
 
   int BasePainter::PaintColorTextLineEdit (GraphicsEngine &GfxContext, const Geometry &g, const NString &Str,
@@ -227,7 +228,7 @@ namespace nux
     tex_h = (float) texture->GetHeight();
 
     TexCoordXForm texxform;
-    GfxContext.QRP_1Tex (x, y, tex_w, tex_h, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (x, y, tex_w, tex_h, texture->GetDeviceTexture(), texxform, color::White);
   }
 
   void BasePainter::Draw2DTextureAligned (GraphicsEngine &GfxContext, BaseTexture *texture, const Geometry &g, TextureAlignmentStyle tex_align) const
@@ -429,12 +430,12 @@ namespace nux
     t_u32 current_green_mask;
     t_u32 current_blue_mask;
     t_u32 current_alpha_mask;
-    
+
     // Get the current color mask and blend states. They will be restored later.
     GfxContext.GetRenderStates ().GetColorMask (current_red_mask, current_green_mask, current_blue_mask, current_alpha_mask);
     GfxContext.GetRenderStates ().GetBlend (current_alpha_blend, current_src_blend_factor, current_dest_blend_factor);
 
-    
+
     GfxContext.GetRenderStates().SetColorMask (GL_TRUE, GL_TRUE, GL_TRUE, WriteAlpha ? GL_TRUE : GL_FALSE);
     GfxContext.GetRenderStates().SetBlend (ROP.Blend, ROP.SrcBlend, ROP.DstBlend);
 
@@ -578,7 +579,7 @@ namespace nux
 
   void BasePainter::PaintTextureShape (GraphicsEngine &GfxContext, const Geometry &geo, BaseTexture *texture,
                                        int border_left, int border_right, int border_top, int border_bottom,
-                                       bool draw_borders_only) const
+                                       bool draw_borders_only, bool premultiply) const
   {
     int tex_w = texture->GetWidth();
     int tex_h = texture->GetHeight();
@@ -598,7 +599,10 @@ namespace nux
       border_top = border_bottom = 0;
     }
 
-    GfxContext.GetRenderStates().SetBlend (TRUE, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    if (premultiply)
+      GfxContext.GetRenderStates().SetBlend (TRUE, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+    else
+      GfxContext.GetRenderStates().SetBlend (TRUE, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
     TexCoordXForm texxform;
     texxform.SetTexCoordType (TexCoordXForm::UNNORMALIZED_COORD);
@@ -608,49 +612,49 @@ namespace nux
     texxform.v0 = 0;
     texxform.u1 = border_left;
     texxform.v1 = border_top;
-    GfxContext.QRP_1Tex (r_x, r_y, border_left, border_top, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x, r_y, border_left, border_top, texture->GetDeviceTexture(), texxform, color::White);
     // Draw TOP-RIGHT CORNER
     texxform.u0 = tex_w - border_right;
     texxform.v0 = 0;
     texxform.u1 = tex_w;
     texxform.v1 = border_top;
-    GfxContext.QRP_1Tex (r_x + r_w - border_right, r_y, border_right, border_top, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x + r_w - border_right, r_y, border_right, border_top, texture->GetDeviceTexture(), texxform, color::White);
     // Draw BOTTOM-LEFT CORNER
     texxform.u0 = 0;
     texxform.v0 = tex_h - border_bottom;
     texxform.u1 = border_left;
     texxform.v1 = tex_h;
-    GfxContext.QRP_1Tex (r_x, r_y + r_h - border_bottom, border_left, border_bottom, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x, r_y + r_h - border_bottom, border_left, border_bottom, texture->GetDeviceTexture(), texxform, color::White);
     // Draw BOTTOM-RIGHT CORNER
     texxform.u0 = tex_w - border_right;
     texxform.v0 = tex_h - border_bottom;
     texxform.u1 = tex_w;
     texxform.v1 = tex_h;
-    GfxContext.QRP_1Tex (r_x + r_w - border_right, r_y + r_h - border_bottom, border_right, border_bottom, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x + r_w - border_right, r_y + r_h - border_bottom, border_right, border_bottom, texture->GetDeviceTexture(), texxform, color::White);
     // Draw TOP BORDER
     texxform.u0 = border_left;
     texxform.v0 = 0;
     texxform.u1 = tex_w - border_right;
     texxform.v1 = border_top;
-    GfxContext.QRP_1Tex (r_x + border_left, r_y, r_w - border_left - border_right, border_top, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x + border_left, r_y, r_w - border_left - border_right, border_top, texture->GetDeviceTexture(), texxform, color::White);
     // Draw BOTTOM BORDER
     texxform.u0 = border_left;
     texxform.v0 = tex_h - border_bottom;
     texxform.u1 = tex_w - border_right;
     texxform.v1 = tex_h;
-    GfxContext.QRP_1Tex (r_x + border_left, r_y + r_h - border_bottom, r_w - border_left - border_right, border_bottom, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x + border_left, r_y + r_h - border_bottom, r_w - border_left - border_right, border_bottom, texture->GetDeviceTexture(), texxform, color::White);
     // Draw LEFT BORDER
     texxform.u0 = 0;
     texxform.v0 = border_top;
     texxform.u1 = border_left;
     texxform.v1 = tex_h - border_bottom;
-    GfxContext.QRP_1Tex (r_x, r_y + border_top, border_left, r_h - border_top - border_bottom, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x, r_y + border_top, border_left, r_h - border_top - border_bottom, texture->GetDeviceTexture(), texxform, color::White);
     // Draw RIGHT BORDER
     texxform.u0 = tex_w - border_right;
     texxform.v0 = border_top;
     texxform.u1 = tex_w;
     texxform.v1 = tex_h - border_bottom;
-    GfxContext.QRP_1Tex (r_x + r_w - border_right, r_y + border_top, border_right, r_h - border_top - border_bottom, texture->GetDeviceTexture(), texxform, Colors::White);
+    GfxContext.QRP_1Tex (r_x + r_w - border_right, r_y + border_top, border_right, r_h - border_top - border_bottom, texture->GetDeviceTexture(), texxform, color::White);
 
     // Draw CENTER
     if (draw_borders_only == false)
@@ -659,7 +663,7 @@ namespace nux
       texxform.v0 = border_top;
       texxform.u1 = tex_w - border_right;
       texxform.v1 = tex_h - border_bottom;
-      GfxContext.QRP_1Tex (r_x + border_left, r_y + border_top, r_w - border_left - border_right, r_h - border_top - border_bottom, texture->GetDeviceTexture(), texxform, Colors::White);
+      GfxContext.QRP_1Tex (r_x + border_left, r_y + border_top, r_w - border_left - border_right, r_h - border_top - border_bottom, texture->GetDeviceTexture(), texxform, color::White);
     }
 
     GfxContext.GetRenderStates().SetBlend (FALSE);
@@ -744,7 +748,7 @@ namespace nux
 
     BaseTexture *texture = pimage->texture;
 
-//     CachedBaseTexture* glr = (CachedBaseTexture*)GetThreadGraphicsContext()->ResourceCache.GetCachedResource(texture);
+//     CachedBaseTexture* glr = (CachedBaseTexture*)GetGraphicsDisplay()->GetGraphicsEngine()->ResourceCache.GetCachedResource(texture);
 //     glr->m_Texture->SetFiltering(GL_LINEAR, GL_LINEAR);
 
     Geometry content_geo (0, 0, texture->GetWidth(), texture->GetHeight() ) ;
@@ -782,7 +786,7 @@ namespace nux
 
   void BasePainter::PaintBackground (GraphicsEngine &GfxContext, const Geometry &geo)
   {
-    if (m_BackgroundStack.begin () == m_BackgroundStack.end ())
+    if (m_BackgroundStack.empty())
     {
       return;
     }
@@ -929,7 +933,7 @@ namespace nux
   {
     nuxAssert (level >= 0);
 
-    while ( (level >= 1) && (m_BackgroundStack.size() > 0 ) )
+    while ((level >= 1) && (m_BackgroundStack.size() > 0))
     {
       AbstractPaintLayer *paint_layer = (*m_BackgroundStack.begin() );
       delete paint_layer;
@@ -940,7 +944,34 @@ namespace nux
 
   void BasePainter::EmptyBackgroundStack()
   {
+    std::list<AbstractPaintLayer*>::iterator background_layer_it;
+    for (background_layer_it = m_BackgroundStack.begin(); background_layer_it != m_BackgroundStack.end(); ++background_layer_it)
+    {
+      delete (*background_layer_it);
+    }
     m_BackgroundStack.clear();
   }
 
+namespace local
+{
+namespace
+{
+  std::list<std::list<AbstractPaintLayer*> > hack_stack;
+}
+}
+
+  void BasePainter::PushBackgroundStack()
+  {
+    local::hack_stack.push_back(m_BackgroundStack);
+    m_BackgroundStack.clear();
+  }
+
+  void BasePainter::PopBackgroundStack()
+  {
+    // clear and delete
+    EmptyBackgroundStack();
+
+    m_BackgroundStack = local::hack_stack.back();
+    local::hack_stack.pop_back();
+  }
 }
