@@ -29,23 +29,23 @@ namespace nux
 {
 
   RGBAPropertyItem::RGBAPropertyItem (const TCHAR *name, float red /* = 1.0f*/, float green /* = 1.0f*/, float blue /* = 1.0f*/, float alpha /* = 1.0f*/)
-    :   SectionProperty (name, NODE_TYPE_RGBA)
-    ,   m_color_model (CM_RGB)
-    ,   m_color_format (Color::COLORFORMAT_FLOAT)
+    : SectionProperty(name, NODE_TYPE_RGBA)
+    , color_(red, green, blue, alpha)
+    , color_model_(color::RGB)
+    , color_format_(color::FLOAT)
   {
     m_red = new ColorGradientPropertyItem (TEXT ("Red") );
     m_green = new ColorGradientPropertyItem (TEXT ("Green") );
     m_blue = new ColorGradientPropertyItem (TEXT ("Blue") );
     m_alpha = new ColorGradientPropertyItem (TEXT ("Alpha") );
 
-    m_ColorModel = new PushButton (TEXT ("RGB"), NUX_TRACKER_LOCATION);
+    m_ColorModel = new ToggleButton ("RGB", NUX_TRACKER_LOCATION);
     m_ColorModel->SetMinMaxSize (32, 14);
     m_ColorModel->SetFont (GetSysBoldFont () );
-    m_ColorFormat = new PushButton (TEXT ("float"), NUX_TRACKER_LOCATION);
+    m_ColorFormat = new ToggleButton ("float", NUX_TRACKER_LOCATION);
     m_ColorFormat->SetMinMaxSize (32, 14);
     m_ColorFormat->SetFont (GetSysBoldFont () );
 
-
     PushChildBack (m_red);
     PushChildBack (m_green);
     PushChildBack (m_blue);
@@ -56,10 +56,10 @@ namespace nux
     float b = Clamp (blue,  0.0f, 1.0f);
     float a = Clamp (alpha, 0.0f, 1.0f);
 
-    m_red->SetColorFormat (m_color_format);
-    m_green->SetColorFormat (m_color_format);
-    m_blue->SetColorFormat (m_color_format);
-    m_alpha->SetColorFormat (m_color_format);
+    m_red->SetColorFormat (color_format_);
+    m_green->SetColorFormat (color_format_);
+    m_blue->SetColorFormat (color_format_);
+    m_alpha->SetColorFormat (color_format_);
 
     m_red->SetRange (0.0f, 1.0f);
     m_red->SetValue (r);
@@ -86,8 +86,8 @@ namespace nux
     m_blue->sigValueChanged.connect ( sigc::mem_fun (this, &RGBAPropertyItem::BlueChange) );
     m_alpha->sigValueChanged.connect ( sigc::mem_fun (this, &RGBAPropertyItem::AlphaChange) );
 
-    m_ColorModel->sigClick.connect (sigc::mem_fun (this, &RGBAPropertyItem::OnChangeColorModel) );
-    m_ColorFormat->sigClick.connect (sigc::mem_fun (this, &RGBAPropertyItem::OnChangeColorFormat) );
+    //FIXME - m_ColorModel->sigClick.connect (sigc::mem_fun (this, &RGBAPropertyItem::OnChangeColorModel) );
+    //FIXME - m_ColorFormat->sigClick.connect (sigc::mem_fun (this, &RGBAPropertyItem::OnChangeColorFormat) );
 
     NODE_SIG_CONNECT (m_red->sigValueChanged, RGBAPropertyItem, RecvPropertyChange);
     NODE_SIG_CONNECT (m_green->sigValueChanged, RGBAPropertyItem, RecvPropertyChange);
@@ -99,20 +99,18 @@ namespace nux
   {
     m_ColorModel->Dispose();
     m_ColorFormat->Dispose();
-    NUX_SAFE_DELETE (m_red);
-    NUX_SAFE_DELETE (m_green);
-    NUX_SAFE_DELETE (m_blue);
-    NUX_SAFE_DELETE (m_alpha);
+    delete m_red;
+    delete m_green;
+    delete m_blue;
+    delete m_alpha;
   }
 
   long RGBAPropertyItem::ProcessPropertyEvent (IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
   {
     long ret = TraverseInfo;
 
-    ret = m_ColorModel->BaseProcessEvent (ievent, ret, ProcessEventInfo);
-    ret = m_ColorFormat->BaseProcessEvent (ievent, ret, ProcessEventInfo);
-//     if(!(ret & eMouseEventSolved))
-//         ret = TableItem::ProcessPropertyEvent(ievent, TraverseInfo, ProcessEventInfo);
+//     ret = m_ColorModel->BaseProcessEvent (ievent, ret, ProcessEventInfo);
+//     ret = m_ColorFormat->BaseProcessEvent (ievent, ret, ProcessEventInfo);
     return ret;
   }
 
@@ -148,9 +146,9 @@ namespace nux
     }
 
     UpdateStartToEndColors();
-    m_green->NeedRedraw();
-    m_blue->NeedRedraw();
-    m_alpha->NeedRedraw();
+    m_green->QueueDraw();
+    m_blue->QueueDraw();
+    m_alpha->QueueDraw();
   }
 
   void RGBAPropertyItem::GreenChange (ColorGradient *slider)
@@ -162,9 +160,9 @@ namespace nux
     }
 
     UpdateStartToEndColors();
-    m_red->NeedRedraw();
-    m_blue->NeedRedraw();
-    m_alpha->NeedRedraw();
+    m_red->QueueDraw();
+    m_blue->QueueDraw();
+    m_alpha->QueueDraw();
   }
 
   void RGBAPropertyItem::BlueChange (ColorGradient *slider)
@@ -176,19 +174,19 @@ namespace nux
     }
 
     UpdateStartToEndColors();
-    m_red->NeedRedraw();
-    m_green->NeedRedraw();
-    m_alpha->NeedRedraw();
+    m_red->QueueDraw();
+    m_green->QueueDraw();
+    m_alpha->QueueDraw();
   }
 
   void RGBAPropertyItem::AlphaChange (ColorGradient *slider)
   {
     UpdateStartToEndColors();
 
-    m_red->NeedRedraw();
-    m_green->NeedRedraw();
-    m_blue->NeedRedraw();
-    m_alpha->NeedRedraw();
+    m_red->QueueDraw();
+    m_green->QueueDraw();
+    m_blue->QueueDraw();
+    m_alpha->QueueDraw();
   }
 
   void RGBAPropertyItem::DrawProperty (GraphicsEngine &GfxContext, TableCtrl *table, bool force_draw, Geometry geo, const BasePainter &Painter,
@@ -211,9 +209,9 @@ namespace nux
         prop_geo.OffsetSize ( - 2 - m_ColorFormat->GetBaseWidth() - 2 - m_ColorModel->GetBaseWidth() - 2, 0);
 
         // Draw the resulting color
-        Painter.Paint2DQuadColor (GfxContext, prop_geo, Color (m_Red, m_Green, m_Blue) );
+        Painter.Paint2DQuadColor (GfxContext, prop_geo, MakeOpaque(color_));
         // Draw black border around the color
-        Painter.Paint2DQuadWireframe (GfxContext, prop_geo, Color (0) );
+        Painter.Paint2DQuadWireframe (GfxContext, prop_geo, color::Black);
         m_ColorModel->ProcessDraw (GfxContext, true);
         m_ColorFormat->ProcessDraw (GfxContext, true);
       }
@@ -224,54 +222,34 @@ namespace nux
 
   void RGBAPropertyItem::OnChangeColorModel()
   {
-    if (m_color_model == CM_RGB)
+    if (color_model_ == color::RGB)
     {
-      SetColorModel (CM_HLS);
-      float H, L, S;
-      RGBtoHLS (m_red->GetValue(), m_green->GetValue(), m_blue->GetValue(), H, L, S);
-      m_red->SetValue (H);
-      m_green->SetValue (L);
-      m_blue->SetValue (S);
+      SetColorModel (color::HLS);
+      color::HueLightnessSaturation hls(color_);
+      SetColor(hls.hue, hls.lightness, hls.saturation, color_.alpha);
     }
-    else if (m_color_model == CM_HLS)
+    else if (color_model_ == color::HLS)
     {
-      SetColorModel (CM_HSV);
-      float H, S, V;
-      float R, G, B;
-      HLStoRGB (R, G, B, m_red->GetValue(), m_green->GetValue(), m_blue->GetValue() );
-      RGBtoHSV (R, G, B, H, S, V);
-      m_red->SetValue (H);
-      m_green->SetValue (S);
-      m_blue->SetValue (V);
-
-      if (H == -1.0f)
-      {
-        H = 0;
-      }
+      SetColorModel (color::HSV);
+      color::HueSaturationValue hsv(color_);
+      SetColor(hsv.hue, hsv.saturation, hsv.value, color_.alpha);
     }
-    else if (m_color_model == CM_HSV)
+    else if (color_model_ == color::HSV)
     {
-      SetColorModel (CM_RGB);
-      float R, G, B;
-      HSVtoRGB (R, G, B, m_red->GetValue(), m_green->GetValue(), m_blue->GetValue() );
-      m_red->SetValue (R);
-      m_green->SetValue (G);
-      m_blue->SetValue (B);
-
+      SetColorModel (color::RGB);
+      SetColor(color_.red, color_.green, color_.blue, color_.alpha);
     }
 
-    UpdateStartToEndColors();
-    m_green->NeedRedraw();
-    m_blue->NeedRedraw();
-    m_alpha->NeedRedraw();
+    m_green->QueueDraw();
+    m_blue->QueueDraw();
   }
 
-  void RGBAPropertyItem::SetColorModel (eColorModel cm)
+  void RGBAPropertyItem::SetColorModel(color::Model cm)
   {
-    if (cm == CM_RGB)
+    if (cm == color::RGB)
     {
-      m_color_model = CM_RGB;
-      m_ColorModel->SetCaption (TEXT ("RGB") );
+      color_model_ = color::RGB;
+      //FIXME - m_ColorModel->SetCaption (TEXT ("RGB") );
 
       m_red->SetName (TEXT ("Red") );
       m_green->SetName (TEXT ("Green") );
@@ -279,10 +257,10 @@ namespace nux
       m_alpha->SetName (TEXT ("Alpha") );
     }
 
-    if (cm == CM_HSV)
+    if (cm == color::HSV)
     {
-      m_color_model = CM_HSV;
-      m_ColorModel->SetCaption (TEXT ("HSV") );
+      color_model_ = color::HSV;
+      //FIXME - m_ColorModel->SetCaption (TEXT ("HSV") );
 
       m_red->SetName (TEXT ("Hue") );
       m_green->SetName (TEXT ("Saturation") );
@@ -290,10 +268,10 @@ namespace nux
       m_alpha->SetName (TEXT ("Alpha") );
     }
 
-    if (cm == CM_HLS)
+    if (cm == color::HLS)
     {
-      m_color_model = CM_HLS;
-      m_ColorModel->SetCaption (TEXT ("HLS") );
+      color_model_ = color::HLS;
+      //FIXME - m_ColorModel->SetCaption (TEXT ("HLS") );
 
       m_red->SetName (TEXT ("Hue") );
       m_green->SetName (TEXT ("Light") );
@@ -301,10 +279,10 @@ namespace nux
       m_alpha->SetName (TEXT ("Alpha") );
     }
 
-    if (cm == CM_YUV)
+    if (cm == color::YUV)
     {
-      m_color_model = CM_YUV;
-      m_ColorModel->SetBaseString (TEXT ("YUV") );
+      color_model_ = color::YUV;
+      //FIXME - m_ColorModel->SetBaseString (TEXT ("YUV") );
 
 //         m_ComponentLabel0->SetBaseString(TEXT("Y"));
 //         m_ComponentLabel1->SetBaseString(TEXT("U"));
@@ -316,26 +294,26 @@ namespace nux
 
   void RGBAPropertyItem::OnChangeColorFormat()
   {
-    if (m_color_format == Color::COLORFORMAT_FLOAT)
+    if (color_format_ == color::FLOAT)
     {
-      m_color_format = Color::COLORFORMAT_INT;
-      m_ColorFormat->SetCaption (TEXT ("int") );
+      color_format_ = color::INT;
+      //FIXME - m_ColorFormat->SetCaption (TEXT ("int") );
     }
-    else if (m_color_format == Color::COLORFORMAT_INT)
+    else if (color_format_ == color::INT)
     {
-      m_color_format = Color::COLORFORMAT_HEX;
-      m_ColorFormat->SetCaption (TEXT ("hex") );
+      color_format_ = color::HEX;
+      //FIXME - m_ColorFormat->SetCaption (TEXT ("hex") );
     }
-    else if (m_color_format == Color::COLORFORMAT_HEX)
+    else if (color_format_ == color::HEX)
     {
-      m_color_format = Color::COLORFORMAT_FLOAT;
-      m_ColorFormat->SetCaption (TEXT ("float") );
+      color_format_ = color::FLOAT;
+      //FIXME - m_ColorFormat->SetCaption (TEXT ("float") );
     }
 
-    m_red->SetColorFormat (m_color_format);
-    m_green->SetColorFormat (m_color_format);
-    m_blue->SetColorFormat (m_color_format);
-    m_alpha->SetColorFormat (m_color_format);
+    m_red->SetColorFormat (color_format_);
+    m_green->SetColorFormat (color_format_);
+    m_blue->SetColorFormat (color_format_);
+    m_alpha->SetColorFormat (color_format_);
   }
 
   void RGBAPropertyItem::UpdateStartToEndColors()
@@ -343,41 +321,27 @@ namespace nux
     m_red->Reset();
     m_green->Reset();
     m_blue->Reset();
-    m_alpha->Reset();
 
-    if (m_color_model == CM_RGB)
+    if (color_model_ == color::RGB)
     {
-      float r, g, b, a;
-      r = m_red->GetValue();
-      g = m_green->GetValue();
-      b = m_blue->GetValue();
-      a = m_alpha->GetValue();
+      color_ = Color(m_red->GetValue(),
+                     m_green->GetValue(),
+                     m_blue->GetValue(),
+                     m_alpha->GetValue());
 
-      m_red->AddColorMark (0, Color (0.0f, g, b), false);
-      m_red->AddColorMark (1, Color (1.0f, g, b), false);
-      m_green->AddColorMark (0, Color (r, 0.0f, b), false);
-      m_green->AddColorMark (1, Color (r, 1.0f, b), false);
-      m_blue->AddColorMark (0, Color (r, g, 0.0f), false);
-      m_blue->AddColorMark (1, Color (r, g, 1.0f), false);
-      m_alpha->AddColorMark (0, Color (0xFF000000), false);
-      m_alpha->AddColorMark (1, Color (0xFFFFFFFF), false);
-
-      m_Red = r;
-      m_Green = g;
-      m_Blue = b;
-      m_Alpha = a;
+      m_red->AddColorMark (0, Color (0.0f, color_.green, color_.blue), false);
+      m_red->AddColorMark (1, Color (1.0f, color_.green, color_.blue), false);
+      m_green->AddColorMark (0, Color (color_.red, 0.0f, color_.blue), false);
+      m_green->AddColorMark (1, Color (color_.red, 1.0f, color_.blue), false);
+      m_blue->AddColorMark (0, Color (color_.red, color_.green, 0.0f), false);
+      m_blue->AddColorMark (1, Color (color_.red, color_.green, 1.0f), false);
     }
 
-    if (m_color_model == CM_HSV)
+    if (color_model_ == color::HSV)
     {
-      float r, g, b, a;
-      float h, s, v;
-      h = m_red->GetValue();
-      s = m_green->GetValue();
-      v = m_blue->GetValue();
-      a = m_alpha->GetValue();
-
-      HSVtoRGB (r, g, b, h, 1.0f, 1.0f);
+      color::HueSaturationValue hsv(m_red->GetValue(),
+                                    m_green->GetValue(),
+                                    m_blue->GetValue());
 
       m_red->AddColorMark (0.0f, Color (1.0f, 0.0, 0.0), false);
       m_red->AddColorMark (1.0f / 6.0f, Color (1.0f, 1.0, 0.0), false);
@@ -387,32 +351,34 @@ namespace nux
       m_red->AddColorMark (5.0f / 6.0f, Color (1.0f, 0.0, 1.0), false);
       m_red->AddColorMark (1.0f, Color (1.0f, 0.0, 0.0), false);
 
-      if (h == 1.0f)
-        h = 0.0f;
+      if (hsv.hue == 1.0f)
+        hsv.hue = 0.0f;
 
-      HSVtoRGB (r, g, b, h, 1.0f, 1.0f);
-      m_green->AddColorMark (0, Color (v, v, v), false);
-      m_green->AddColorMark (1.0f, Color (r * v, g * v, b * v), false);
+      color::RedGreenBlue rgb(hsv);
+      color_ = Color(rgb, m_alpha->GetValue());
 
-      HSVtoRGB (r, g, b, h, s, 1.0f);
-      m_blue->AddColorMark (0, Color (0, 0, 0), false);
-      m_blue->AddColorMark (1.0f, Color (r, g, b), false);
+      // The green holds the saturation.
+      Color min_green(hsv.value, hsv.value, hsv.value);
 
-      m_alpha->AddColorMark (0, Color (0xFF000000), false);
-      m_alpha->AddColorMark (1, Color (0xFFFFFFFF), false);
+      // The blue slider handles full value.
+      hsv.value = 1.0f;
+      color::RedGreenBlue blue_slider(hsv);
+      m_blue->AddColorMark (0, color::Black, false);
+      m_blue->AddColorMark (1.0f, Color(blue_slider), false);
 
-      HSVtoRGB (m_Red, m_Green, m_Blue, h, s, v);
-      m_Alpha = a;
+      // Max green slider has full saturation and value
+      hsv.saturation = 1.0f;
+      color::RedGreenBlue green_slider(hsv);
+      Color max_green = Color(green_slider) * hsv.value;
+      m_green->AddColorMark (0, min_green, false);
+      m_green->AddColorMark (1.0f, max_green, false);
     }
 
-    if (m_color_model == CM_HLS)
+    if (color_model_ == color::HLS)
     {
-      float r, g, b, a;
-      float h, l, s;
-      h = m_red->GetValue();
-      l = m_green->GetValue();
-      s = m_blue->GetValue();
-      a = m_alpha->GetValue();
+      color::HueLightnessSaturation hls(m_red->GetValue(),
+                                        m_green->GetValue(),
+                                        m_blue->GetValue());
 
       m_red->AddColorMark (0.0f, Color (1.0f, 0.0, 0.0), false);
       m_red->AddColorMark (1.0f / 6.0f, Color (1.0f, 1.0, 0.0), false);
@@ -422,43 +388,38 @@ namespace nux
       m_red->AddColorMark (5.0f / 6.0f, Color (1.0f, 0.0, 1.0), false);
       m_red->AddColorMark (1.0f, Color (1.0f, 0.0, 0.0), false);
 
-      s = 1.0f - s;
+      if (hls.hue == 1.0f)
+        hls.hue = 0.0f;
 
-      if (h == 1.0f)
-        h = 0.0f;
+      color::RedGreenBlue rgb(hls);
+      color_ = Color(rgb, m_alpha->GetValue());
+
+      float s = (1.0f - hls.saturation) * 0.5;
 
       // Need to use HSVtoRGB to compute the primary color
-      HSVtoRGB (r, g, b, h, 1.0f, 1.0f);
-      m_green->AddColorMark (0.0f, Color (0, 0, 0), false);
-      m_green->AddColorMark (0.5f, Color (r* (1 - s) + 0.5f * s, g* (1 - s) + 0.5f * s, b* (1 - s) + 0.5f * s), false);
-      m_green->AddColorMark (1.0f, Color (1.0f, 1.0f, 1.0f), false);
+      color::HueSaturationValue primary_hsv(hls.hue, 1.0f, 1.0f);
+      color::RedGreenBlue primary_rgb(primary_hsv);
+      Color primary = Color(primary_rgb) * hls.saturation + s;
+      m_green->AddColorMark (0.0f, color::Black, false);
+      m_green->AddColorMark (0.5f, primary, false);
+      m_green->AddColorMark (1.0f, color::White, false);
 
-      float cr, cg, cb;
+      // Not sure on the name of this color...
+      Color secondary = Color(primary_rgb);
 
-      if (l > 0.5)
+      if (hls.lightness > 0.5)
       {
-        float factor = (l - 0.5f) / 0.5f;
-        cr = (1 - factor) * r * (1 - s) + 0.5 * s + factor * 1.0f;
-        cg = (1 - factor) * g * (1 - s) + 0.5 * s + factor * 1.0f;
-        cb = (1 - factor) * b * (1 - s) + 0.5 * s + factor * 1.0f;
+        float factor = (hls.lightness - 0.5f) / 0.5f;
+        secondary = secondary * ((1 - factor) * hls.saturation) + (s + factor);
       }
       else
       {
-        float factor = l / 0.5f;
-        cr = (factor) * r * (1 - s) + 0.5 * s;
-        cg = (factor) * g * (1 - s) + 0.5 * s;
-        cb = (factor) * b * (1 - s) + 0.5 * s;
+        float factor = hls.lightness / 0.5f * hls.saturation;
+        secondary = secondary * factor + s;
       }
 
-      m_blue->AddColorMark (0, Color (l, l, l), false);
-      m_blue->AddColorMark (1.0f, Color (cr, cg, cb), false);
-
-
-      m_alpha->AddColorMark (0, Color (0xFF000000), false);
-      m_alpha->AddColorMark (1, Color (0xFFFFFFFF), false);
-
-      HLStoRGB (m_Red, m_Green, m_Blue, h, l, 1.0f - s);
-      m_Alpha = a;
+      m_blue->AddColorMark (0, Color (hls.lightness, hls.lightness, hls.lightness), false);
+      m_blue->AddColorMark (1.0f, secondary, false);
     }
   }
 
@@ -491,19 +452,19 @@ namespace nux
     TiXmlElement *childxml;
     childxml = new TiXmlElement (TEXT ("RGBAComponent") );
     //childxml->SetAttribute(TEXT("Name"), m_X->GetName());
-    childxml->SetDoubleAttribute (TEXT ("Red"), m_Red);
+    childxml->SetDoubleAttribute (TEXT ("Red"), color_.red);
     elementxml->LinkEndChild (childxml);
     childxml = new TiXmlElement (TEXT ("RGBAComponent") );
     //childxml->SetAttribute(TEXT("Name"), m_Y->GetName());
-    childxml->SetDoubleAttribute (TEXT ("Green"), m_Green);
+    childxml->SetDoubleAttribute (TEXT ("Green"), color_.green);
     elementxml->LinkEndChild (childxml);
     childxml = new TiXmlElement (TEXT ("RGBAComponent") );
     //childxml->SetAttribute(TEXT("Name"), m_Z->GetName());
-    childxml->SetDoubleAttribute (TEXT ("Blue"), m_Blue);
+    childxml->SetDoubleAttribute (TEXT ("Blue"), color_.blue);
     elementxml->LinkEndChild (childxml);
     childxml = new TiXmlElement (TEXT ("RGBAComponent") );
     //childxml->SetAttribute(TEXT("Name"), m_W->GetName());
-    childxml->SetDoubleAttribute (TEXT ("Alpha"), m_Alpha);
+    childxml->SetDoubleAttribute (TEXT ("Alpha"), color_.alpha);
     elementxml->LinkEndChild (childxml);
 
     return elementxml;
