@@ -56,18 +56,18 @@ namespace nux
     m_CubicSpline.Set (m_control_knot.GetNumKnot(), m_control_knot.GetXArray(), m_control_knot.GetYArray() );
 
 
-    m_Texture = GetGpuDevice()->CreateSystemCapableDeviceTexture (256, 4, 0, BITFMT_R8G8B8A8);
+    m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableDeviceTexture (256, 4, 0, BITFMT_R8G8B8A8);
     m_DrawFunctionShader = new GLSh_DrawFunction();
 
     NTextureData image;
     MakeCheckBoardImage (image.GetSurface (0), 64, 64, Color (0xff323232), Color (0xff535353), 8, 8);
-    BaseTexture* CheckboardPattern = GetGpuDevice()->CreateSystemCapableTexture ();
+    BaseTexture* CheckboardPattern = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture ();
     CheckboardPattern->Update(&image);
 
     TexCoordXForm texxform;
     texxform.SetTexCoordType (TexCoordXForm::OFFSET_COORD);
     texxform.SetWrap (TEXWRAP_REPEAT, TEXWRAP_REPEAT);
-    m_Background = new TextureLayer (CheckboardPattern->GetDeviceTexture(), texxform, Colors::White);
+    m_Background = new TextureLayer (CheckboardPattern->GetDeviceTexture(), texxform, color::White);
 
     CheckboardPattern->UnReference ();;
 //     m_Background = PaintLayer(m_CheckboardPattern);
@@ -76,7 +76,6 @@ namespace nux
 
   SplineCurveEditor::~SplineCurveEditor()
   {
-    DestroyLayout();
     NUX_SAFE_DELETE (m_DrawFunctionShader);
     NUX_SAFE_DELETE (m_Background);
   }
@@ -89,14 +88,10 @@ namespace nux
   void SplineCurveEditor::InitializeLayout()
   {
     SetMinimumSize (200, 200);
-    OnMouseDown.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvMouseDown) );
-    OnMouseUp.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvMouseUp) );
-    OnMouseDrag.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvMouseDrag) );
-    OnKeyEvent.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvKeyEvent) );
-  }
-
-  void SplineCurveEditor::DestroyLayout()
-  {
+    mouse_down.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvMouseDown) );
+    mouse_up.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvMouseUp) );
+    mouse_drag.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvMouseDrag) );
+    key_down.connect (sigc::mem_fun (this, &SplineCurveEditor::RecvKeyEvent) );
   }
 
   void SplineCurveEditor::SetControlPoints (const SplineKnot &splineKnot)
@@ -116,7 +111,7 @@ namespace nux
     m_control_knot.AddKnot (x, y, selected);
     m_CubicSpline.Set (m_control_knot.GetNumKnot(), m_control_knot.GetXArray(), m_control_knot.GetYArray() );
     sigCurveChange.emit (this);
-    NeedRedraw();
+    QueueDraw();
   }
 
   void SplineCurveEditor::Reset()
@@ -251,7 +246,7 @@ namespace nux
         m_Texture->UnlockRect (0);
 
         m_DrawFunctionShader->SetTextureFunction (m_Texture);
-        m_DrawFunctionShader->SetBackgroundColor (0.1f, 0.1f, 0.1f, 0.6f);
+        m_DrawFunctionShader->SetBackgroundColor (Color(0.1f, 0.1f, 0.1f, 0.6f));
         m_DrawFunctionShader->Render (X, Y, 0, W, H, GfxContext.GetWindowWidth(), GfxContext.GetWindowHeight() );
         //m_DrawFunctionShader->End();
 
@@ -309,8 +304,8 @@ namespace nux
 
       GfxContext.PopClippingRectangle();
 
-      delete t;
-      delete y;
+      delete[] t;
+      delete[] y;
     }
 
     // We do some ajustment here because when a knot is at one of the border, we still want to see the entire knot and not have the wireframe
@@ -337,20 +332,20 @@ namespace nux
   {
     m_minX = minX;
     m_maxX = maxX;
-    NeedRedraw();
+    QueueDraw();
   }
 
   void SplineCurveEditor::SetYAxisBounds (float minY, float maxY)
   {
     m_minY = minY;
     m_maxY = maxY;
-    NeedRedraw();
+    QueueDraw();
   }
 
   void SplineCurveEditor::SetFunctionCallback (SplineFunctionCallback f)
   {
     m_FunctionCallback = f;
-    NeedRedraw();
+    QueueDraw();
   }
 
   float SplineCurveEditor::EvalFunction (float x)
@@ -363,7 +358,7 @@ namespace nux
 
   void SplineCurveEditor::UpdateGraph()
   {
-    NeedRedraw();
+    QueueDraw();
   }
 
 
@@ -379,7 +374,7 @@ namespace nux
 
   void SplineCurveEditor::RecvMouseUp (int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
-    NeedRedraw();
+    QueueDraw();
 
     if (m_control_knot.GetNumSelectedKnot() > 0)
       sigCurveChange.emit (this);
@@ -431,7 +426,7 @@ namespace nux
 
         m_control_knot.SelectKnot (i, TRUE);
         m_CubicSpline.Set (m_control_knot.GetNumKnot(), m_control_knot.GetXArray(), m_control_knot.GetYArray() );
-        NeedRedraw();
+        QueueDraw();
         return;
       }
     }
@@ -443,7 +438,7 @@ namespace nux
       m_control_knot.AddKnot (new_x, new_y, TRUE);
       m_CubicSpline.Set (m_control_knot.GetNumKnot(), m_control_knot.GetXArray(), m_control_knot.GetYArray() );
       sigCurveChange.emit (this);
-      NeedRedraw();
+      QueueDraw();
     }
   }
 
@@ -576,11 +571,10 @@ namespace nux
     }
 
     m_CubicSpline.Set (m_control_knot.GetNumKnot(), m_control_knot.GetXArray(), m_control_knot.GetYArray() );
-    NeedRedraw();
+    QueueDraw();
   }
 
   void SplineCurveEditor::RecvKeyEvent (
-    GraphicsEngine  &GfxContext , /*Graphics Context for text operation*/
     unsigned long   eventType  , /*event type*/
     unsigned long   keysym     , /*event keysym*/
     unsigned long   state      , /*event state*/
@@ -604,7 +598,7 @@ namespace nux
       if (kn > 0)
         sigCurveChange.emit (this);
 
-      NeedRedraw();
+      QueueDraw();
     }
   }
 
