@@ -44,17 +44,17 @@ namespace nux
   {
     _index = 0;
     _tab_name = TabName;
-    _tab_layout = TabLayout;
+    _tab_content_layout = TabLayout;
     _tab_area = new InputArea (NUX_TRACKER_LOCATION);
 
-    _tab_layout->Reference();
+    _tab_content_layout->Reference();
     _tab_area->Reference();
   }
 
   TabView::TabElement::~TabElement()
   {
     _index = 0;
-    _tab_layout->UnReference();
+    _tab_content_layout->UnReference();
     _tab_area->UnReference();
   }
 
@@ -73,7 +73,7 @@ namespace nux
     _tab_area->SetGeometry (geo);
   }
 
-  Geometry TabView::TabElement::GetGeometry() const
+  Geometry const& TabView::TabElement::GetGeometry() const
   {
     return _tab_area->GetGeometry();
   }
@@ -86,38 +86,40 @@ namespace nux
   TabView::TabView (NUX_FILE_LINE_DECL)
     :   View (NUX_FILE_LINE_PARAM)
   {
-    m_IncrTab           = NULL;
-    m_DecrTab           = NULL;
-    m_ClientLayout      = NULL;
-    m_TabLayout         = NULL;
-    m_TabControlLayout  = NULL;
+    _scroll_right           = NULL;
+    _scroll_left           = NULL;
+    _visible_tab_content_layout      = NULL;
+    _tabview_heads_layout         = NULL;
+    _tabview_scroll_button_layout  = NULL;
 
     m_FocusTabIndex     = 0;
     m_TabPositionOffset = 0;
     m_DrawBackgroundOnPreviousGeometry = false;
 
-    m_TabLayout         = new HLayout (NUX_TRACKER_LOCATION);
-    m_TabControlLayout  = new HLayout (NUX_TRACKER_LOCATION);
-    m_IncrTab           = new InputArea (NUX_TRACKER_LOCATION);
-    m_DecrTab           = new InputArea (NUX_TRACKER_LOCATION);
+    _tabview_heads_layout         = new HLayout (NUX_TRACKER_LOCATION);
+    //_tabview_heads_layout->SetParentObject(this);
+    _tabview_scroll_button_layout  = new HLayout (NUX_TRACKER_LOCATION);
+    _tabview_scroll_button_layout->SetParentObject(this);
+    _scroll_right           = new InputArea (NUX_TRACKER_LOCATION);
+    _scroll_left           = new InputArea (NUX_TRACKER_LOCATION);
 
-    m_IncrTab->SetMinimumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
-    m_DecrTab->SetMinimumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
-    m_IncrTab->SetMaximumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
-    m_DecrTab->SetMaximumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
+    _scroll_right->SetMinimumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
+    _scroll_left->SetMinimumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
+    _scroll_right->SetMaximumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
+    _scroll_left->SetMaximumSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
 
-    m_IncrTab->OnMouseEnter.connect (sigc::mem_fun (this, &TabView::RecvMouseEnter) );
-    m_IncrTab->OnMouseLeave.connect (sigc::mem_fun (this, &TabView::RecvMouseLeave) );
-    m_DecrTab->OnMouseEnter.connect (sigc::mem_fun (this, &TabView::RecvMouseEnter) );
-    m_DecrTab->OnMouseLeave.connect (sigc::mem_fun (this, &TabView::RecvMouseLeave) );
+    _scroll_right->mouse_enter.connect (sigc::mem_fun (this, &TabView::RecvMouseEnter) );
+    _scroll_right->mouse_leave.connect (sigc::mem_fun (this, &TabView::RecvMouseLeave) );
+    _scroll_left->mouse_enter.connect (sigc::mem_fun (this, &TabView::RecvMouseEnter) );
+    _scroll_left->mouse_leave.connect (sigc::mem_fun (this, &TabView::RecvMouseLeave) );
 
-    m_IncrTab->OnMouseDown.connect (sigc::mem_fun (this, &TabView::RecvTabRightMouseDown) );
-    m_DecrTab->OnMouseDown.connect (sigc::mem_fun (this, &TabView::RecvTabLeftMouseDown) );
-    m_IncrTab->OnMouseUp.connect (sigc::mem_fun (this, &TabView::RecvTabButtonMouseUp) );
-    m_DecrTab->OnMouseUp.connect (sigc::mem_fun (this, &TabView::RecvTabButtonMouseUp) );
+    _scroll_right->mouse_down.connect (sigc::mem_fun (this, &TabView::RecvTabRightMouseDown) );
+    _scroll_left->mouse_down.connect (sigc::mem_fun (this, &TabView::RecvTabLeftMouseDown) );
+    _scroll_right->mouse_up.connect (sigc::mem_fun (this, &TabView::RecvTabButtonMouseUp) );
+    _scroll_left->mouse_up.connect (sigc::mem_fun (this, &TabView::RecvTabButtonMouseUp) );
 
-    m_TabControlLayout->AddView (m_DecrTab, 1, eCenter);
-    m_TabControlLayout->AddView (m_IncrTab, 1, eCenter);
+    _tabview_scroll_button_layout->AddView (_scroll_left, 1, eCenter);
+    _tabview_scroll_button_layout->AddView (_scroll_right, 1, eCenter);
 
 
     tabright_callback = new TimerFunctor;
@@ -128,48 +130,77 @@ namespace nux
 
   TabView::~TabView()
   {
-    m_TabLayout->Dispose();
-    m_TabControlLayout->Dispose();
+    _tabview_heads_layout->UnReference();
+    _tabview_scroll_button_layout->UnReference();
     RemoveCompositionLayout();
     delete (tabright_callback);
     delete (tableft_callback);
     
     std::vector<TabElement *>::iterator it;
 
-    for (it = m_TabVector.begin(); it != m_TabVector.end(); it++)
+    for (it = _tab_array.begin(); it != _tab_array.end(); it++)
     {
+      (*it)->_tab_area->UnParentObject();
+      (*it)->_tab_content_layout->UnParentObject();
       delete (*it);
     }
 
-    m_TabVector.clear();
+    _tab_array.clear();
   }
 
   long TabView::ProcessEvent (IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
   {
     long ret = TraverseInfo;
 
-    ret = m_IncrTab->OnEvent (ievent, ret, ProcessEventInfo);
-    ret = m_DecrTab->OnEvent (ievent, ret, ProcessEventInfo);
+    ret = _scroll_right->OnEvent (ievent, ret, ProcessEventInfo);
+    ret = _scroll_left->OnEvent (ievent, ret, ProcessEventInfo);
 
-//    if(m_IncrTab->HasMouseFocus())
+//    if(_scroll_right->IsMouseOwner())
 //        TranslateTabLayout(-10);
-//    if(m_DecrTab->HasMouseFocus())
+//    if(_scroll_left->IsMouseOwner())
 //        TranslateTabLayout(10);
 
-    t_u32 vector_size = (t_u32) m_TabVector.size();
+    t_u32 vector_size = (t_u32) _tab_array.size();
 
     for (t_u32 i = 0; i < vector_size; i++)
     {
-      ret = m_TabVector[i]->_tab_area->OnEvent (ievent, ret, ProcessEventInfo);
+      ret = _tab_array[i]->_tab_area->OnEvent (ievent, ret, ProcessEventInfo);
     }
 
-    if (m_ClientLayout)
-      ret = m_ClientLayout->ProcessEvent (ievent, ret, ProcessEventInfo);
+    if (_visible_tab_content_layout)
+      ret = _visible_tab_content_layout->ProcessEvent (ievent, ret, ProcessEventInfo);
 
     ret = PostProcessEvent2 (ievent, ret, 0);
     return ret;
   }
 
+  Area* TabView::FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type)
+  {
+    bool mouse_inside = TestMousePointerInclusionFilterMouseWheel(mouse_position, event_type);
+
+    if(mouse_inside == false)
+      return NULL;
+
+    NUX_RETURN_VALUE_IF_TRUE(_scroll_right->TestMousePointerInclusion(mouse_position, event_type), _scroll_right);
+    NUX_RETURN_VALUE_IF_TRUE(_scroll_left->TestMousePointerInclusion(mouse_position, event_type), _scroll_left);
+
+    t_u32 vector_size = (t_u32) _tab_array.size();
+    for(t_u32 i = 0; i < vector_size; i++)
+    {
+      NUX_RETURN_VALUE_IF_TRUE(_tab_array[i]->_tab_area->TestMousePointerInclusion(mouse_position, event_type), _tab_array[i]->_tab_area);
+    }
+
+    if(_visible_tab_content_layout)
+    {
+      nuxAssert(_visible_tab_content_layout->IsLayout());
+      Area* found_area = _visible_tab_content_layout->FindAreaUnderMouse(mouse_position, event_type);
+      if(found_area)
+        return found_area;
+    }
+
+    return this;
+  }
+
 
   void TabView::Draw (GraphicsEngine &GfxContext, bool force_draw)
   {
@@ -184,26 +215,26 @@ namespace nux
 
     GetPainter().PushDrawShapeLayer (GfxContext, Geometry (base.x, base.y, base.GetWidth(), TAB_HEIGHT), eSHAPE_CORNER_ROUND4, TAB_HEADER_BACKGROUND_COLOR, eCornerTopLeft | eCornerTopRight);
 
-    if (m_ClientLayout)
-      m_ClientLayout->NeedRedraw();
+    if (_visible_tab_content_layout)
+      _visible_tab_content_layout->QueueDraw();
 
-    t_u32 vector_size = (t_u32) m_TabVector.size();
+    t_u32 vector_size = (t_u32) _tab_array.size();
 
     Geometry geo = GetGeometry();
     Geometry clip_geo;
     clip_geo.SetX (geo.x);
     clip_geo.SetY (geo.y);
     clip_geo.SetWidth (geo.GetWidth() - 2 * TAB_BUTTON_WIDTH);
-    clip_geo.SetHeight (m_TabLayout->GetBaseHeight() );
+    clip_geo.SetHeight (_tabview_heads_layout->GetBaseHeight() );
 
     GfxContext.PushClippingRectangle (clip_geo);
 
     for (t_u32 i = 0; i < vector_size; i++)
     {
-      Geometry tab_geo = m_TabVector[i]->_tab_area->GetGeometry();
-      const char *tab_text = m_TabVector[i]->GetName().GetTCharPtr();
+      Geometry tab_geo = _tab_array[i]->_tab_area->GetGeometry();
+      const char *tab_text = _tab_array[i]->GetName().GetTCharPtr();
 
-      if (m_TabVector[i]->_index == m_FocusTabIndex)
+      if (_tab_array[i]->_index == m_FocusTabIndex)
       {
         tab_geo.OffsetSize (-2, 0);
         GetPainter().PaintShapeCorner (GfxContext, tab_geo, TAB_HEADER_FOCUS_COLOR, eSHAPE_CORNER_ROUND4, eCornerTopLeft | eCornerTopRight, false);
@@ -222,20 +253,20 @@ namespace nux
     GetPainter().PaintShapeCorner (GfxContext, Geometry (base.x, base.y + TAB_HEIGHT, base.GetWidth(), base.GetHeight() - TAB_HEIGHT),
                                TAB_BACKGROUND_COLOR, eSHAPE_CORNER_ROUND4, eCornerBottomLeft | eCornerBottomRight, false);
 
-    GetPainter().Paint2DQuadColor (GfxContext, m_IncrTab->GetGeometry(), TAB_HEADER_BACKGROUND_COLOR);
+    GetPainter().Paint2DQuadColor (GfxContext, _scroll_right->GetGeometry(), TAB_HEADER_BACKGROUND_COLOR);
     GeometryPositioning gp (eHACenter, eVACenter);
-    Geometry GeoPo = ComputeGeometryPositioning (m_IncrTab->GetGeometry(), GetTheme().GetImageGeometry (eTAB_RIGHT), gp);
+    Geometry GeoPo = ComputeGeometryPositioning (_scroll_right->GetGeometry(), GetTheme().GetImageGeometry (eTAB_RIGHT), gp);
 
-    if (m_IncrTab->IsMouseInside() )
+    if (_scroll_right->IsMouseInside() )
       GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eTAB_RIGHT);
     else
       GetPainter().PaintShape (GfxContext, GeoPo, TAB_HEADER_FOCUS_COLOR, eTAB_RIGHT);
 
-    GetPainter().Paint2DQuadColor (GfxContext, m_DecrTab->GetGeometry(), TAB_HEADER_BACKGROUND_COLOR);
+    GetPainter().Paint2DQuadColor (GfxContext, _scroll_left->GetGeometry(), TAB_HEADER_BACKGROUND_COLOR);
     gp.SetAlignment (eHACenter, eVACenter);
-    GeoPo = ComputeGeometryPositioning (m_DecrTab->GetGeometry(), GetTheme().GetImageGeometry (eTAB_LEFT), gp);
+    GeoPo = ComputeGeometryPositioning (_scroll_left->GetGeometry(), GetTheme().GetImageGeometry (eTAB_LEFT), gp);
 
-    if (m_DecrTab->IsMouseInside() )
+    if (_scroll_left->IsMouseInside() )
       GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eTAB_LEFT);
     else
       GetPainter().PaintShape (GfxContext, GeoPo, TAB_HEADER_FOCUS_COLOR, eTAB_LEFT);
@@ -252,10 +283,10 @@ namespace nux
     GetPainter().PushShapeLayer (GfxContext, Geometry (base.x, base.y + TAB_HEIGHT, base.GetWidth(), base.GetHeight() - TAB_HEIGHT),
                              eSHAPE_CORNER_ROUND4, TAB_BACKGROUND_COLOR, eCornerBottomLeft | eCornerBottomRight);
 
-    if (m_ClientLayout)
+    if (_visible_tab_content_layout)
     {
-      GfxContext.PushClippingRectangle (m_ClientLayout->GetGeometry() );
-      m_ClientLayout->ProcessDraw (GfxContext, force_draw);
+      GfxContext.PushClippingRectangle (_visible_tab_content_layout->GetGeometry() );
+      _visible_tab_content_layout->ProcessDraw (GfxContext, force_draw);
       GfxContext.PopClippingRectangle();
     }
 
@@ -298,24 +329,24 @@ namespace nux
     int tab_x = m_CompositionLayout->GetGeometry().x + m_CompositionLayout->GetGeometry().GetWidth() - 2 * TAB_BUTTON_WIDTH;
     int tab_y = base.y;
 
-    m_DecrTab->SetBaseXY (tab_x, tab_y);
-    m_DecrTab->SetBaseSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
-    m_IncrTab->SetBaseXY (tab_x + TAB_BUTTON_WIDTH, tab_y);
-    m_IncrTab->SetBaseSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
+    _scroll_left->SetBaseXY (tab_x, tab_y);
+    _scroll_left->SetBaseSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
+    _scroll_right->SetBaseXY (tab_x + TAB_BUTTON_WIDTH, tab_y);
+    _scroll_right->SetBaseSize (TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT);
 
-    m_TabLayout->SetBaseY (tab_y);
-    m_TabLayout->SetBaseHeight (TAB_HEIGHT);
+    _tabview_heads_layout->SetBaseY (tab_y);
+    _tabview_heads_layout->SetBaseHeight (TAB_HEIGHT);
     TranslateTabLayout (0);
 
-    m_TabControlLayout->SetBaseXY (tab_x, tab_y);
-    m_TabControlLayout->SetBaseSize (2 * TAB_BUTTON_WIDTH, TAB_HEIGHT);
-    GetWindowThread ()->ComputeElementLayout (m_TabControlLayout);
+    _tabview_scroll_button_layout->SetBaseXY (tab_x, tab_y);
+    _tabview_scroll_button_layout->SetBaseSize (2 * TAB_BUTTON_WIDTH, TAB_HEIGHT);
+    GetWindowThread ()->ComputeElementLayout (_tabview_scroll_button_layout);
 
-    if (m_ClientLayout)
+    if (_visible_tab_content_layout)
     {
-//        m_ClientLayout->SetGeometry(m_CompositionLayout->GetGeometry());
-//        GetWindowThread ()->ComputeElementLayout(m_ClientLayout);
-      m_ClientLayout->NeedRedraw();
+//        _visible_tab_content_layout->SetGeometry(m_CompositionLayout->GetGeometry());
+//        GetWindowThread ()->ComputeElementLayout(_visible_tab_content_layout);
+      _visible_tab_content_layout->QueueDraw();
     }
 
     return LayoutResult;
@@ -332,57 +363,58 @@ namespace nux
   }
 
 
-  void TabView::AddTab (const char *tab_name, Layout *tab_layout)
+  void TabView::AddTab(const char *tab_name, Layout *tab_layout)
   {
     if (tab_layout == 0)
       return;
 
     TabElement *Tab = new TabElement (tab_name, tab_layout);
-    Tab->SetIndex (m_TabVector.size() );
+
+    Tab->SetIndex (_tab_array.size() );
 
     if (Tab->GetIndex() == 0)
     {
       m_FocusTabIndex = 0;
-      m_ClientLayout = Tab->_tab_layout;
-      //m_ClientLayout->SetGeometry(m_CompositionLayout->GetGeometry());
-      SetCompositionLayout (m_ClientLayout);
+      _visible_tab_content_layout = Tab->_tab_content_layout;
+      //_visible_tab_content_layout->SetGeometry(m_CompositionLayout->GetGeometry());
+      SetCompositionLayout (_visible_tab_content_layout);
       GetWindowThread ()->ComputeElementLayout (this);
     }
 
     Tab->_tab_area->SetMinimumSize (6 + GetSysBoldFont()->GetStringWidth (tab_name), PRACTICAL_WIDGET_HEIGHT);
     Tab->_tab_area->SetMaximumSize (6 + GetSysBoldFont()->GetStringWidth (tab_name), PRACTICAL_WIDGET_HEIGHT);
 
-    Tab->_tab_area->OnMouseDown.connect (sigc::bind ( sigc::mem_fun (this, &TabView::RecvTabMouseDown), Tab) );
-    Tab->_tab_area->OnMouseUp.connect (sigc::bind ( sigc::mem_fun (this, &TabView::RecvTabMouseUp), Tab) );
+    Tab->_tab_area->mouse_down.connect (sigc::bind ( sigc::mem_fun (this, &TabView::RecvTabMouseDown), Tab) );
+    Tab->_tab_area->mouse_up.connect (sigc::bind ( sigc::mem_fun (this, &TabView::RecvTabMouseUp), Tab) );
 
-    m_TabLayout->AddView (Tab->_tab_area, 1, eCenter);
-    GetWindowThread ()->ComputeElementLayout (m_TabLayout);
+    _tabview_heads_layout->AddView (Tab->_tab_area, 1, eCenter);
+    GetWindowThread ()->ComputeElementLayout (_tabview_heads_layout);
 
-    m_TabVector.push_back (Tab);
+    _tab_array.push_back (Tab);
   }
 
   void TabView::SetActiveTad (int index)
   {
-    if (index >= (t_s32) m_TabVector.size() )
+    if (index >= (t_s32) _tab_array.size() )
     {
-      m_FocusTabIndex = (t_s32) m_TabVector.size() - 1;
+      m_FocusTabIndex = (t_s32) _tab_array.size() - 1;
     }
     else
     {
       m_FocusTabIndex = index;
     }
 
-    m_ClientLayout = m_TabVector[m_FocusTabIndex]->_tab_layout;
+    _visible_tab_content_layout = _tab_array[m_FocusTabIndex]->_tab_content_layout;
 
-    if (m_ClientLayout)
-      SetCompositionLayout (m_ClientLayout);
+    if (_visible_tab_content_layout)
+      SetCompositionLayout (_visible_tab_content_layout);
 
     GetWindowThread ()->ComputeElementLayout (this);
 
     sigTabChanged (this);
     sigTabIndexChanged (m_FocusTabIndex);
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TabView::TranslateLeft (int x, int y, unsigned long button_flags, unsigned long key_flags)
@@ -410,12 +442,12 @@ namespace nux
       lx = m_CompositionLayout->GetBaseX() + m_TabPositionOffset;
     }
 
-    if (lx + m_TabLayout->GetBaseWidth() < m_CompositionLayout->GetBaseX() +
+    if (lx + _tabview_heads_layout->GetBaseWidth() < m_CompositionLayout->GetBaseX() +
         m_CompositionLayout->GetBaseWidth() - 2 * TAB_BUTTON_WIDTH)
     {
 
       lx = (m_CompositionLayout->GetBaseX() +
-            m_CompositionLayout->GetBaseWidth() - 2 * TAB_BUTTON_WIDTH) - m_TabLayout->GetBaseWidth();
+            m_CompositionLayout->GetBaseWidth() - 2 * TAB_BUTTON_WIDTH) - _tabview_heads_layout->GetBaseWidth();
 
 
       if (lx > m_CompositionLayout->GetBaseX() )
@@ -427,23 +459,23 @@ namespace nux
         m_TabPositionOffset = - (m_CompositionLayout->GetBaseX() - lx);
     }
 
-    m_TabLayout->SetBaseX (lx);
-    GetWindowThread ()->ComputeElementLayout (m_TabLayout);
-    NeedRedraw();
+    _tabview_heads_layout->SetBaseX (lx);
+    GetWindowThread ()->ComputeElementLayout (_tabview_heads_layout);
+    QueueDraw();
 
   }
 
   void TabView::RecvTabMouseDown (int x, int y, unsigned long button_flags, unsigned long key_flags, TabElement *tab)
   {
     m_FocusTabIndex = tab->_index;
-    m_ClientLayout = tab->_tab_layout;
+    _visible_tab_content_layout = tab->_tab_content_layout;
 
-    if (m_ClientLayout)
+    if (_visible_tab_content_layout)
     {
-      SetCompositionLayout (m_ClientLayout);
+      SetCompositionLayout (_visible_tab_content_layout);
     }
 
-    //m_ClientLayout->SetGeometry(m_CompositionLayout->GetGeometry());
+    //_visible_tab_content_layout->SetGeometry(m_CompositionLayout->GetGeometry());
 
     m_PreviousGeometry = GetGeometry();
 
@@ -469,7 +501,7 @@ namespace nux
 
     sigTabChanged (this);
     sigTabIndexChanged (m_FocusTabIndex);
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TabView::RecvTabMouseUp (int x, int y, unsigned long button_flags, unsigned long key_flags, TabElement *tab)
@@ -479,12 +511,12 @@ namespace nux
 
   void TabView::RecvMouseEnter (int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TabView::RecvMouseLeave (int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TabView::RecvTabRightMouseDown (int x, int y, unsigned long button_flags, unsigned long key_flags)
@@ -521,4 +553,8 @@ namespace nux
     m_TabLeftTimerHandler = GetTimer().AddTimerHandler (10, tableft_callback, this);
   }
 
+  bool TabView::AcceptKeyNavFocus()
+  {
+    return false;
+  }
 }
