@@ -91,12 +91,12 @@ namespace nux
     m_TableItemHead->Show();
 
     m_TableArea = new InputArea (NUX_TRACKER_LOCATION);
-    m_TableArea->OnMouseDown.connect (sigc::mem_fun (this, &TableCtrl::OnMouseDown) );
-    m_TableArea->OnMouseDoubleClick.connect (sigc::mem_fun (this, &TableCtrl::OnMouseDoubleClick) );
-    m_TableArea->OnMouseUp.connect (sigc::mem_fun (this, &TableCtrl::OnMouseUp) );
-    m_TableArea->OnMouseDrag.connect (sigc::mem_fun (this, &TableCtrl::OnMouseDrag) );
-    m_TableArea->OnStartFocus.connect (sigc::mem_fun (this, &TableCtrl::OnKeyboardFocus) );
-    m_TableArea->OnEndFocus.connect (sigc::mem_fun (this, &TableCtrl::OnLostKeyboardFocus) );
+    m_TableArea->mouse_down.connect (sigc::mem_fun (this, &TableCtrl::OnMouseDown) );
+    m_TableArea->mouse_double_click.connect (sigc::mem_fun (this, &TableCtrl::mouse_double_click) );
+    m_TableArea->mouse_up.connect (sigc::mem_fun (this, &TableCtrl::OnMouseUp) );
+    m_TableArea->mouse_drag.connect (sigc::mem_fun (this, &TableCtrl::mouse_drag) );
+    m_TableArea->begin_key_focus.connect (sigc::mem_fun (this, &TableCtrl::OnKeyboardFocus) );
+    m_TableArea->end_key_focus.connect (sigc::mem_fun (this, &TableCtrl::OnLostKeyboardFocus) );
 
 
 
@@ -130,8 +130,8 @@ namespace nux
     SetMinimumSize (50, 50);
     SetGeometry (Geometry (0, 0, 50, 50) );
 
-    setTopBorder (0);
-    setBorder (0);
+    m_top_border = 0;
+    m_border = 0;
 
     SetViewContentRightMargin (2);
     SetViewContentBottomMargin (2);
@@ -151,11 +151,6 @@ namespace nux
     std::vector<RowHeader *>::iterator it2;
     std::vector<sizehandler2 *>::iterator it3;
 
-//    for(it1 = m_column_header.begin(); it1 != m_column_header.end(); it1++)
-//    {
-//        delete (*it1);
-//    }
-
     EmptyTable();
 
     for (it2 = m_row_header.begin(); it2 != m_row_header.end(); it2++)
@@ -247,7 +242,7 @@ namespace nux
           if (ret & eMouseEventSolved)
             ItemSolvedEvent = true;
 
-          // Pure TableItem elements do not inherit from View. Therefore, they cannot call NeedRedraw() on themselves.
+          // Pure TableItem elements do not inherit from View. Therefore, they cannot call QueueDraw() on themselves.
           // The TableCtrl that holds them must know that they need to be redrawn. Since Pure TableItem do not trap events, the event
           // will go to TableCtrl (like OnMouseDown) who will found out the cell where the event happened and set the dirty flag of the TableItem.
           //if((*row_iterator)->_table_item->isDirtyItem())
@@ -257,13 +252,13 @@ namespace nux
     }
 
 //    if(DirtyItem)
-//        NeedRedraw();
+//        QueueDraw();
 
     if (m_vertical_scrollbar_enable)
-      ret = vscrollbar->ProcessEvent (ievent, ret, ProcEvInfo & (~eDoNotProcess) );
+      ret = _vscrollbar->ProcessEvent (ievent, ret, ProcEvInfo & (~eDoNotProcess) );
 
     if (m_horizontal_scrollbar_enable)
-      ret = hscrollbar->ProcessEvent (ievent, ret, ProcEvInfo & (~eDoNotProcess) );
+      ret = _hscrollbar->ProcessEvent (ievent, ret, ProcEvInfo & (~eDoNotProcess) );
 
     if (IsPartOfCombobox() && ievent.e_event == NUX_MOUSE_RELEASED)
     {
@@ -335,7 +330,7 @@ namespace nux
         {
           m_selectedColumn = -1;
           m_selectedRow = -1;
-          NeedRedraw();
+          QueueDraw();
         }
       }
     }
@@ -345,7 +340,7 @@ namespace nux
       {
         m_selectedColumn = -1;
         m_selectedRow = -1;
-        NeedRedraw();
+        QueueDraw();
       }
     }
 
@@ -379,7 +374,7 @@ namespace nux
       texxform0.uwrap = TEXWRAP_CLAMP;
       texxform0.vwrap = TEXWRAP_CLAMP;
       texxform0.FlipVCoord (true);
-      GetThreadGraphicsContext()->QRP_1Tex (x, y, w, h, texture, texxform0, Colors::White);
+      GetGraphicsDisplay()->GetGraphicsEngine()->QRP_1Tex (x, y, w, h, texture, texxform0, color::White);
     }
   }
 
@@ -410,12 +405,12 @@ namespace nux
 
     if (m_vertical_scrollbar_enable)
     {
-      vscrollbar->NeedRedraw();
+      _vscrollbar->QueueDraw();
     }
 
     if (m_horizontal_scrollbar_enable)
     {
-      hscrollbar->NeedRedraw();
+      _hscrollbar->QueueDraw();
     }
 
     DrawTable (GfxContext);
@@ -485,12 +480,12 @@ namespace nux
 
     if (m_vertical_scrollbar_enable)
     {
-      vscrollbar->ProcessDraw (GfxContext, force_draw);
+      _vscrollbar->ProcessDraw (GfxContext, force_draw);
     }
 
     if (m_horizontal_scrollbar_enable)
     {
-      hscrollbar->ProcessDraw (GfxContext, force_draw);
+      _hscrollbar->ProcessDraw (GfxContext, force_draw);
     }
 
     GfxContext.PopClippingRectangle();
@@ -753,7 +748,7 @@ namespace nux
     ObjectPtr<IOpenGLFrameBufferObject> CurrentFrameBuffer = GetWindowCompositor().GetWindowFrameBufferObject();
 
     Geometry tableGeometry = m_TableArea->GetGeometry();
-    int xl, yl, wl, hl;
+    int xl, yl;
 
     //--->GetPainter().Paint2DQuadColor(tableGeometry, Color(HEADER_BACKGROUND_COLOR));
 
@@ -790,8 +785,6 @@ namespace nux
       // Render row name and horizontal separation
       xl = m_TableArea->GetGeometry().x;
       yl = m_TableArea->GetGeometry().y + (m_bShowColumnHeader ? COLUMNHEADERHEIGHT : 0); // skip the Height of the Column header;
-      wl = m_TableArea->GetGeometry().GetWidth();
-      hl = m_TableArea->GetGeometry().GetHeight();
 
       // Paint the first horizontal separation line.
       GetPainter().Draw2DLine (GfxContext, xl, yl, xl + m_ContentGeometry.GetWidth(), yl, GetHorizontalSeparationLineColor() );
@@ -871,12 +864,10 @@ namespace nux
       //GetPainter().PaintShapeCorner(GfxContext, header_geo, 0xFF2f2f2f, eSHAPE_CORNER_ROUND10, eCornerTopLeft|eCornerTopRight);
     }
 
-    int xl, yl, wl, hl;
+    int xl, yl;
     // Render header name and vertical separations
     xl = m_TableArea->GetGeometry().x + (m_bShowRowHeader ? ROWHEADERWIDTH : 0); // skip the width of the Row header
     yl = m_TableArea->GetGeometry().y + (m_bShowColumnHeader ? COLUMNHEADERHEIGHT : 0);
-    wl = m_TableArea->GetGeometry().GetWidth();
-    hl = m_TableArea->GetGeometry().GetHeight() - (m_bShowColumnHeader ? COLUMNHEADERHEIGHT : 0);
 
     // Paint the first vertical separation line.
     if (m_bShowRowHeader == false)
@@ -1213,9 +1204,9 @@ namespace nux
     column_sizehandler->SetBaseHeight ( (m_bShowColumnHeader ? COLUMNHEADERHEIGHT : 0) );
 
     t_u32 no = (t_u32) m_column_sizehandler.size();
-    column_sizehandler->OnMouseDown.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeHeaderMouseDown), no) );
-    column_sizehandler->OnMouseUp.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeHeaderMouseUp), no) );
-    column_sizehandler->OnMouseDrag.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeHeaderMouseDrag), no) );
+    column_sizehandler->mouse_down.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeHeaderMouseDown), no) );
+    column_sizehandler->mouse_up.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeHeaderMouseUp), no) );
+    column_sizehandler->mouse_drag.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeHeaderMouseDrag), no) );
     m_column_sizehandler.push_back (column_sizehandler);
 
 
@@ -1473,8 +1464,8 @@ namespace nux
       row_sizehandler->SetBaseHeight (HANDLERSIZE);
 
       t_u32 row_no = (t_u32) m_row_sizehandler.size();
-      row_sizehandler->OnMouseDown.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeRowMouseDown), row_no) );
-      row_sizehandler->OnMouseDrag.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeRowMouseDrag), row_no) );
+      row_sizehandler->mouse_down.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeRowMouseDown), row_no) );
+      row_sizehandler->mouse_drag.connect (sigc::bind ( sigc::mem_fun (this, &TableCtrl::OnResizeRowMouseDrag), row_no) );
       m_row_sizehandler.push_back (row_sizehandler);
 
 
@@ -1709,9 +1700,6 @@ namespace nux
   {
     std::vector<Geometry>::iterator it;
 
-    // selected item geometry
-    int sx, sy, sw, sh;
-
     row = -1;
     t_u32 num_row = (t_u32) m_row_header.size();
 
@@ -1724,8 +1712,6 @@ namespace nux
              (m_row_header[i]->_table_item->_row_header_area->GetBaseY() + m_row_header[i]->_table_item->_row_header_area->GetBaseHeight() > y) )
         {
           row = i;
-          sy = m_row_header[i]->_table_item->_row_header_area->GetBaseY();
-          sh = m_row_header[i]->_table_item->_row_header_area->GetBaseHeight();
           break;
         }
       }
@@ -1739,8 +1725,6 @@ namespace nux
       if ( (m_column_header[i].m_header_area->GetBaseX() <= x) && (m_column_header[i].m_header_area->GetBaseX() + m_column_header[i].m_header_area->GetBaseWidth() > x) )
       {
         column = i;
-        sx = m_column_header[i].m_header_area->GetBaseX();
-        sw = m_column_header[i].m_header_area->GetBaseWidth();
         break;
       }
     }
@@ -1757,7 +1741,7 @@ namespace nux
   {
     if (m_selectedTableItem)
     {
-      // Dirty the previously selected item. No need to call NeedRedraw because this is a mouse down and the
+      // Dirty the previously selected item. No need to call QueueDraw because this is a mouse down and the
       // WindowThread will call Draw(event, false) on a NUX_MOUSE_PRESSED event.
       // When this item redraws, it will draw without the selection background color.
       m_selectedTableItem->setDirtyItem (true);
@@ -1784,7 +1768,7 @@ namespace nux
       sigItemSelected.emit (m_selectedRow, m_selectedColumn);
       sigTableItemSelected.emit (*this, *m_selectedTableItem, m_selectedRow, m_selectedColumn);
 
-      // Dirty the newly selected item. No need to call NeedRedraw because this is a mouse down and the
+      // Dirty the newly selected item. No need to call QueueDraw because this is a mouse down and the
       // WindowThread will call Draw(event, false) on a NUX_MOUSE_PRESSED event.
       // When this item redraws, it will draw with the selection background color.
       m_selectedTableItem->setDirtyItem (true);
@@ -1836,10 +1820,10 @@ namespace nux
     if (Reorganize == true)
     {
       // The TableCtrl is completely dirty.
-      // By calling NeedRedraw(), we make sure we go through TableCtrl::Draw() then TableCtrl::DrawContent() when drawing happens.
+      // By calling QueueDraw(), we make sure we go through TableCtrl::Draw() then TableCtrl::DrawContent() when drawing happens.
       ComputeChildLayout();
       PositionChildLayout (0, 0);
-      NeedRedraw();
+      QueueDraw();
     }
   }
 
@@ -1857,7 +1841,7 @@ namespace nux
 //(5-12), and you should begin to notice that Windows is no longer ignoring as many double-clicks
 //as it did before.
 
-  void TableCtrl::OnMouseDoubleClick (int x, int y, unsigned long button_flags, unsigned long key_flags)
+  void TableCtrl::mouse_double_click (int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
     if ( (m_selectedRow == -1) || (m_selectedColumn == -1) )
     {
@@ -1945,7 +1929,7 @@ namespace nux
     {
       FormatTable();
       ComputeChildLayout();
-      NeedRedraw();
+      QueueDraw();
     }
   }
 
@@ -1976,7 +1960,7 @@ namespace nux
     }
   }
 
-  void TableCtrl::OnMouseDrag (int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
+  void TableCtrl::mouse_drag (int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
   {
     return;
 
@@ -1988,17 +1972,10 @@ namespace nux
 
     if ( (m_selectedRow != -1) && (m_selectedColumn != -1) )
     {
-      // selected item geometry
-      int sx, sy, sw, sh;
-      sx = m_column_header[m_selectedColumn].m_header_area->GetBaseX();
-      sw = m_column_header[m_selectedColumn].m_header_area->GetBaseWidth();
-      sy = m_row_header[m_selectedRow]->_table_item->_row_header_area->GetBaseY();
-      sh = m_row_header[m_selectedRow]->_table_item->_row_header_area->GetBaseHeight();
-
       m_selectedTableItem->setDirtyItem (true);
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TableCtrl::OnKeyboardFocus()
@@ -2013,7 +1990,7 @@ namespace nux
   void TableCtrl::OnResizeHeaderMouseDown (int x, int y, unsigned long button_flags, unsigned long key_flags, t_u32 header_pos)
   {
     //m_column_header_preview = m_column_header;
-    m_point0.Set (x, y);
+    m_point0 = Point(x, y);
   }
 
   void TableCtrl::OnResizeHeaderMouseUp (int x, int y, unsigned long button_flags, unsigned long key_flags, t_u32 header_pos)
@@ -2028,7 +2005,7 @@ namespace nux
       //ComputeChildLayout();
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TableCtrl::OnResizeHeaderMouseDrag (int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags, t_u32 header_pos)
@@ -2080,7 +2057,6 @@ namespace nux
     if (isFloatingColumn() == false)
     {
       // Need to find the next element who can absorb the difference
-      bool findElement = false;
       int element_pos;
       int deltax = x - m_point0.x;
 
@@ -2110,7 +2086,6 @@ namespace nux
 
         if ( (m_column_header[i]._fix_width == false) /*&& (element_width > 10)*/)
         {
-          findElement = true;
           element_pos = i;
 
           if ( (dx > 0) && (x > m_point0.x) )
@@ -2190,12 +2165,12 @@ namespace nux
       FormatHeader();
     }
 
-    //NeedRedraw();
+    //QueueDraw();
   }
 
   void TableCtrl::OnResizeRowMouseDown (int x, int y, unsigned long button_flags, unsigned long key_flags, t_u32 header_pos)
   {
-    m_point1.Set (x, y);
+    m_point1 = Point(x, y);
 
     char s[42];
     sprintf (s, "%d ", header_pos);
@@ -2232,7 +2207,7 @@ namespace nux
       ComputeChildLayout();
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void TableCtrl::PreLayoutManagement()
@@ -2378,20 +2353,20 @@ namespace nux
   void TableCtrl::EmptyTable()
   {
     m_TableItemHead->DeleteTree();
-    NeedRedraw();
+    QueueDraw();
   }
 
-  void TableCtrl::NeedRedraw()
+  void TableCtrl::QueueDraw()
   {
     nuxAssertNoRecursion();
-    View::NeedRedraw();
+    View::QueueDraw();
   }
 
   void TableCtrl::HighlightItem (int row, int column)
   {
     m_selectedRow = row;
     m_selectedColumn = column;
-    NeedRedraw();
+    QueueDraw();
   }
 
 
