@@ -101,11 +101,8 @@ namespace nux
     {
       return *m_TimerHandler;
     }
-    
-    UXTheme &GetTheme() const
-    {
-      return *m_Theme;
-    }
+
+    UXTheme& GetTheme() const;
 
     void SetWindowSize (int width, int height);
 
@@ -265,6 +262,8 @@ namespace nux
 
     */
     virtual bool ThreadCtor (Display *X11Display, Window X11Window, GLXContext OpenGLContext);
+    Display *_x11display;
+    bool     _ownx11display;
 #endif
 
     /*!
