@@ -24,7 +24,6 @@
 #define GLRESOURCE_H
 
 #include "NuxCore/NuxCore.h"
-#include "NuxCore/Namespace.h"
 #include "NuxCore/SystemTypes.h"
 #include "NuxCore/Error.h"
 #include "NuxCore/FilePath.h"
@@ -35,7 +34,6 @@
 #include "NuxImage/BitmapFormats.h"
 #include "NuxCore/Parsing.h"
 #include "NuxCore/Object.h"
-#include "NuxCore/SmartPtr/GenericSmartPointer.h"
 #include "NuxCore/ObjectPtr.h"
 
 #include "NuxCore/Math/MathUtility.h"
@@ -60,7 +58,7 @@ namespace nux
   class TextureVolume;
   class TextureFrameAnimation;
 
-  class NVertexBuffer;
+  class BaseMeshBuffer;
   class NIndexBuffer;
 
   class CachedTexture2D;
@@ -74,8 +72,6 @@ namespace nux
 
 #define NUX_ENABLE_CG_SHADERS 0
 
-//#define NUX_OPENGLES_20
-
 #if defined(NUX_OS_WINDOWS)
   #include "GL/glew.h"
   #include "GL/wglew.h"
@@ -93,11 +89,15 @@ namespace nux
 #elif defined(NUX_OS_LINUX)
 
   #ifdef NUX_OPENGLES_20
-    #define GLEW_MX
+    #ifndef GLEW_MX
+      #define GLEW_MX
+    #endif
     #include "EGL/egl.h"
     #include "GLES2/gl2.h"
   #else
-    #define GLEW_MX
+    #ifndef GLEW_MX
+      #define GLEW_MX
+    #endif
     #include "GL/glew.h"
     #include "GL/glxew.h"
     
@@ -247,7 +247,7 @@ namespace nux
     PRIMITIVE_TYPE_FORCE_DWORD           = 0x7fffffff /* force 32-bit size enum */
   } PRIMITIVE_TYPE;
 
-  typedef enum
+  enum OpenGLResourceType
   {
     RTINDEXBUFFER,
     RTVERTEXBUFFER,
@@ -264,11 +264,12 @@ namespace nux
     RTFRAMEBUFFEROBJECT,
     RT_GLSL_VERTEXSHADER,
     RT_GLSL_PIXELSHADER,
+    RT_GLSL_GEOMETRYSHADER,
     RT_GLSL_SHADERPROGRAM,
     RT_CG_VERTEXSHADER,
     RT_CG_PIXELSHADER,
     RT_FORCE_DWORD           = 0x7fffffff /* force 32-bit size enum */
-  } OpenGLResourceType;
+  };
 
   /* Multi-Sample buffer types */
   typedef enum
@@ -494,53 +495,37 @@ namespace nux
       Offset = 0;
       Type = ATTRIB_CT_UNKNOWN;
       NumComponent = 0;
-      Usage = ATTRIB_USAGE_DECL_POSITION;
-      UsageIndex = 0;
-      Method = 0;
+      stride_ = 0;
     }
 
-    VERTEXELEMENT (WORD stream, WORD offset, ATTRIB_COMPONENT_TYPE type, BYTE numcomponents, ATTRIB_USAGE_DECL usage, BYTE usageindex, BYTE method = 0)
+    VERTEXELEMENT(
+      int stream,
+      int offset,
+      ATTRIB_COMPONENT_TYPE type,
+      BYTE numcomponents,
+      int stride)
     {
       Stream = stream;
       Offset = offset;
       Type = type;
       NumComponent = numcomponents;
-      Usage = usage;
-      UsageIndex = usageindex;
-      Method = method;
+      stride_ = stride;
     }
 
-    WORD Stream;
-    WORD Offset;
-
+    int Stream;
+    int Offset;
     // Type can be GL_UNSIGNED_BYTE, GL_SHORT, GL_INT, GL_FLOAT, GL_DOUBLE ...
     ATTRIB_COMPONENT_TYPE Type;
-    //DWORD Stride;
-
     // This can be 1, 2, 3 or 4; For a position (xyzw), it will be 4. For a texture coordinate (uv) it will be 2.
-    BYTE NumComponent;
-
-    ATTRIB_USAGE_DECL Usage;
-
-    // Use this index in association with TEXCOORD or COLOR to identify the texture
-    // coordinate set or the color set.
-    BYTE UsageIndex;
-
-    BYTE Method; // Should be always 0 for D3DDECLMETHOD_DEFAULT = 0
-
-    // The vertex attribute Cg parameter that will be bound to the vertex element;
-//    CGparameter CgParameter;
-//    VERTEXELEMENT()
-//    {
-//        CgParameter = 0;
-//    }
-//    void setCgParameter(CGparameter parameter)
-//    {
-//        cgParameter = parameter;
-//    }
+    int NumComponent;
+    int stride_;
   };
 
-#define DECL_END VERTEXELEMENT(0xFF/*Stream*/,0/*Offset*/,ATTRIB_CT_UNKNOWN/*Type*/,0/*NumComponent*/,ATTRIB_USAGE_DECL_POSITION/*Usage*/,0/*UsageIndex*/, 0 /*CgParameter*/)
+#define DECL_END VERTEXELEMENT( \
+  0xFF,                       \
+  0,                          \
+  ATTRIB_CT_UNKNOWN,          \
+  0, 0)
 
   unsigned int GetVertexElementSize (VERTEXELEMENT vtxelement);
 
@@ -549,333 +534,16 @@ namespace nux
 
   void DecomposeTypeDeclaraction (ATTRIB_DECL_TYPE Type, BYTE &NumComponent, ATTRIB_COMPONENT_TYPE &ComponentType);
 
-  void AddVertexElement (std::vector<VERTEXELEMENT>& Elements,
-                         WORD Stream,
-                         WORD Offset,
-                         //ubiS16 Stride,
-                         ATTRIB_DECL_TYPE Type,
-                         ATTRIB_USAGE_DECL Usage,
-                         BYTE UsageIndex);
-
-  GLenum GetGLPrimitiveType (PRIMITIVE_TYPE InPrimitiveType);
-  unsigned int   GetGLElementCount (PRIMITIVE_TYPE InPrimitiveType,
-                                    unsigned int        InPrimitiveCount);
-
-//   template<typename T> class ObjectPtr
-//   {
-//   public:
-//     T	*Handle;
-// 
-//   private:
-//     void CheckReferenceObject()
-//     {
-//       if (Handle)
-//       {
-//         if (Handle->GetValue() == 0)
-//         {
-//           delete Handle;
-//           //GetGpuDevice()->DestroyDeviceResource<T>(Handle);
-//         }
-//       }
-// 
-//       Handle = 0;
-//     }
-// 
-//     //     // Do not allow access to the managed pointer. This also avoid compiler confusion between
-//     //     //  int BindTexture(IOpenGLBaseTexture* texture);
-//     //     // and
-//     //     //  int BindTexture(ObjectPtr<IOpenGLBaseTexture> texture);
-//     //
-//     //     // Access operators.
-//     //     typedef	T*	PtrT;
-//     //     operator T*()
-//     //     {
-//     //         return Handle;
-//     //     }
-// 
-//   public:
-//     T *operator -> ()
-//     {
-//       nuxAssert (Handle);
-//       return Handle;
-//     }
-// 
-//     const T *operator -> () const
-//     {
-//       nuxAssert (Handle);
-//       return Handle;
-//     }
-// 
-//     T &operator*()
-//     {
-//       nuxAssert (Handle);
-//       return *Handle;
-//     }
-// 
-//     const T &operator *() const
-//     {
-//       nuxAssert (Handle);
-//       return *Handle;
-//     }
-// 
-//     // Constructor/destructor.
-//     ObjectPtr (T *InHandle = 0)
-//       :   Handle (InHandle)
-//     {
-//       if (Handle)
-//         Handle->Increment();
-//     }
-// 
-//     template <typename U>
-//     ObjectPtr (U *InHandle)
-//       :   Handle (0)
-//     {
-//       if (InHandle == 0)
-//         return;
-// 
-//       if (InHandle->Type().IsDerivedFromType (T::StaticObjectType) )
-//       {
-//         Handle = (T *) InHandle;
-//       }
-// 
-//       if (Handle)
-//         Handle->Increment();
-//     }
-// 
-//     ObjectPtr (const ObjectPtr<T>& Copy)
-//       :   Handle (0)
-//     {
-//       Handle = Copy.Handle;
-// 
-//       if (Handle)
-//         Handle->Increment();
-//     }
-// 
-//     template <typename U>
-//     ObjectPtr (const ObjectPtr<U>& Copy)
-//       :   Handle (0)
-//     {
-//       // Check if type U is derived from type T
-//       // Type() is virtual. Even if we have a ObjectPtr<IOpenGLBaseTexture> but its internal pointer is a IOpenGLCubeTexture,
-//       // Type() returns the static object type inside IOpenGLCubeTexture.
-//       // This make this possible
-//       //
-//       //      ObjectPtr<IOpenGLBaseTexture> basetex;            // IOpenGLBaseTexture is an abstract base class for IOpenGLTexture2D
-//       //      basetex = gGLDeviceFactory->CreateTexture(.....);
-//       //      ObjectPtr<IOpenGLVertexBuffer> vtx(basetex);   // vtx Handle will be null
-//       //      ObjectPtr<IOpenGLTexture2D> tex(basetex);      // tex Handle will be the same as basetex
-//       //
-// 
-//       if (Copy.Handle->Type().IsDerivedFromType (T::StaticObjectType) )
-//       {
-//         Handle = (T *) Copy.Handle;
-//       }
-// 
-//       if (Handle)
-//         Handle->Increment();
-//     }
-// 
-//     ~ObjectPtr()
-//     {
-//       if (Handle)
-//       {
-//         Handle->Decrement();
-//         CheckReferenceObject();
-//       }
-//     }
-// 
-//     // Assignment operator.
-// 
-//     ObjectPtr<T>& operator = (T *InHandle)
-//     {
-//       if (Handle != InHandle)
-//       {
-//         if (Handle)
-//         {
-//           Handle->Decrement();
-//           CheckReferenceObject();
-//         }
-// 
-//         Handle = InHandle;
-// 
-//         if (Handle)
-//           Handle->Increment();
-//       }
-// 
-//       return *this;
-//     }
-// 
-//     ObjectPtr<T>& operator = (const ObjectPtr<T>& Other)
-//     {
-//       // Avoid self assignment
-//       if (Handle == Other.Handle)
-//       {
-//         return *this;
-//       }
-// 
-//       if (Handle)
-//       {
-//         Handle->Decrement();
-//         CheckReferenceObject();
-//       }
-// 
-//       Handle = Other.Handle;
-// 
-//       if (Handle)
-//         Handle->Increment();
-// 
-//       return *this;
-//     }
-// 
-//     bool operator == (const ObjectPtr<T>& Other) const
-//     {
-//       if (Handle == Other.Handle)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     bool operator == (const T *Ptr) const
-//     {
-//       if (Handle == Ptr)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     template <typename U>
-//     bool operator == (const U *Ptr) const
-//     {
-//       if (Handle == Ptr)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     template <typename U>
-//     bool operator == (ObjectPtr<U>& Other) const
-//     {
-//       if (Handle == Other.Handle)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     bool operator != (const ObjectPtr<T>& Other) const
-//     {
-//       if (Handle != Other.Handle)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     bool operator != (const T *Ptr) const
-//     {
-//       if (Handle != Ptr)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     template <typename U>
-//     bool operator != (const U *Ptr) const
-//     {
-//       if (Handle != Ptr)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     template <typename U>
-//     bool operator != (ObjectPtr<U>& Other) const
-//     {
-//       if (Handle != Other.Handle)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     //    // This isn't safe. An assignment like the following is wrong, but it will no flag an error at compile time.
-//     //    // ObjectPtr<IOpenGLVertexBuffer> vtx;
-//     //    // ObjectPtr<IOpenGLTexture2D> tex;
-//     //    // vtx = tex;         // WRONG
-//     //
-//     //    template<typename U>
-//     //        ObjectPtr<T>& operator=(const ObjectPtr<U>& Other)
-//     //    {
-//     //        Handle = (T*)(Other.Handle);
-//     //        return *this;
-//     //    }
-// 
-//     // Doing vtx = tex.Castref<IOpenGLVertexBuffer> will return a ObjectPtr<IOpenGLVertexBuffer> with a null Handle.
-//     template<typename U>
-//     ObjectPtr<U> CastRef()
-//     {
-//       ObjectPtr<U> t;
-// 
-//       // Check if type U is derived from type T
-//       if (U::StaticObjectType.IsDerivedFromType (T::StaticObjectType) )
-//       {
-//         t = (U *) Handle;
-//       }
-// 
-//       return t;
-//     }
-// 
-//     void Release()
-//     {
-//       if (Handle)
-//       {
-//         Handle->Decrement();
-//         CheckReferenceObject();
-//       }
-// 
-//       Handle = 0;
-//     }
-//   private:
-//     T &GetHandle()
-//     {
-//       return *Handle;
-//     }
-// 
-//   public:
-//     bool IsValid()
-//     {
-//       if (Handle)
-//       {
-//         return true;
-//       }
-// 
-//       return false;
-//     }
-// 
-//     bool IsNull()
-//     {
-//       if (Handle)
-//       {
-//         return false;
-//       }
-// 
-//       return true;
-//     }
-//   };
+//   void AddVertexElement (std::vector<VERTEXELEMENT>& Elements,
+//                          WORD Stream,
+//                          WORD Offset,
+//                          //ubiS16 Stride,
+//                          ATTRIB_DECL_TYPE Type,
+//                          ATTRIB_USAGE_DECL Usage,
+//                          BYTE UsageIndex);
 
+  GLenum GetGLPrimitiveType(PRIMITIVE_TYPE InPrimitiveType);
+  unsigned int GetGLElementCount(PRIMITIVE_TYPE InPrimitiveType, unsigned int InPrimitiveCount);
 }
 
 #endif // GLRESOURCE_H
