@@ -22,6 +22,7 @@
 
 #include "GLDeviceObjects.h"
 #include "IOpenGLVolume.h"
+#include "GraphicsDisplay.h"
 #include "GpuDevice.h"
 
 namespace nux
@@ -72,13 +73,8 @@ namespace nux
 
 
     IOpenGLVolumeTexture *texture = _VolumeTexture;
-    OpenGLResourceType ResourceType = RTBASETEXTURE;
 
-    if (_VolumeTexture->_ResourceType == RTVOLUMETEXTURE)
-    {
-      ResourceType = RTVOLUMETEXTURE;
-    }
-    else
+    if (_VolumeTexture->_ResourceType != RTVOLUMETEXTURE)
     {
       nuxAssertMsg (0, TEXT ("Unknown resource type") );
     }
@@ -162,25 +158,25 @@ namespace nux
     _Box.Front   = 0;
     _Box.Back    = ImageSurface::GetLevelDim (texture->_PixelFormat, texture->GetDepth(), _SMipLevel);
 
-    if (GetGpuDevice()->UsePixelBufferObjects() )
+    if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
     {
-      GetGpuDevice()->AllocateUnpackPixelBufferIndex (&_AllocatedUnpackBuffer);
+      GetGraphicsDisplay()->GetGpuDevice()->AllocateUnpackPixelBufferIndex (&_AllocatedUnpackBuffer);
     }
 
     if (pBox == 0)
     {
       _CompressedDataSize = GetDepth() * surface_size;
 
-      if (GetGpuDevice()->UsePixelBufferObjects() )
+      if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
       {
         // Mapping the entire area of the surface
         if (1)
         {
-          _LockedBox.pBits = GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, _CompressedDataSize);
+          _LockedBox.pBits = GetGraphicsDisplay()->GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, _CompressedDataSize);
         }
         else
         {
-          GetGpuDevice()->BindUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
+          GetGraphicsDisplay()->GetGpuDevice()->BindUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
           CHECKGL ( glBufferDataARB (GL_PIXEL_UNPACK_BUFFER_ARB, _CompressedDataSize, NULL, GL_STREAM_DRAW_ARB) );
           _LockedBox.pBits = glMapBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB, GL_WRITE_ONLY_ARB);
           CHECKGL_MSG (glMapBufferARB );
@@ -228,11 +224,11 @@ namespace nux
 
       _CompressedDataSize = NumSlice * RectSize;
 
-      if (GetGpuDevice()->UsePixelBufferObjects() )
+      if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
       {
-        _LockedBox.pBits = GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, NumSlice * RectSize);
+        _LockedBox.pBits = GetGraphicsDisplay()->GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, NumSlice * RectSize);
 
-//             GetGpuDevice()->BindUnpackPixelBufferIndex(_AllocatedUnpackBuffer);
+//             GetGraphicsDisplay()->GetGpuDevice()->BindUnpackPixelBufferIndex(_AllocatedUnpackBuffer);
 //             CHECKGL( glBufferDataARB(GL_PIXEL_UNPACK_BUFFER_ARB, NumSlice * RectSize, NULL, GL_STATIC_DRAW_ARB) );
 //             _LockedBox.pBits = glMapBufferARB(GL_PIXEL_UNPACK_BUFFER_ARB, GL_WRITE_ONLY);
 //             CHECKGL_MSG(glMapBufferARB );
@@ -279,10 +275,10 @@ namespace nux
     {
       CHECKGL ( glBindTexture (_STextureTarget, _VolumeTexture->_OpenGLID) );
 
-      if (GetGpuDevice()->UsePixelBufferObjects() )
+      if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
       {
         // Unmap the texture image buffer
-        GetGpuDevice()->BindUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
+        GetGraphicsDisplay()->GetGpuDevice()->BindUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
         CHECKGL ( glUnmapBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB) );
         DataPtr = NUX_BUFFER_OFFSET (0);
       }
@@ -357,10 +353,10 @@ namespace nux
       nuxAssertMsg (0, TEXT ("Incorrect Texture Target.") );
     }
 
-    if (GetGpuDevice()->UsePixelBufferObjects() )
+    if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
     {
       CHECKGL ( glBindBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB, 0) );
-      GetGpuDevice()->FreeUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
+      GetGraphicsDisplay()->GetGpuDevice()->FreeUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
     }
     else
     {
@@ -371,7 +367,7 @@ namespace nux
       }
     }
 
-    CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, GetGpuDevice()->GetPixelStoreAlignment() ) );
+    CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, GetGraphicsDisplay()->GetGpuDevice()->GetPixelStoreAlignment() ) );
 
     _LockedBox.pBits = 0;
     _LockedBox.RowPitch = 0;
@@ -463,7 +459,7 @@ namespace nux
     //        delete [] color_array;
     //    }
 
-    CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, GetGpuDevice()->GetPixelStoreAlignment() ) );
+    CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, GetGraphicsDisplay()->GetGpuDevice()->GetPixelStoreAlignment() ) );
 
     _Initialized = true;
     return OGL_OK;
