@@ -21,32 +21,38 @@
 
 
 #include "NuxCore/NuxCore.h"
-#include "NuxCore/Color.h"
-#include "ImageSurface.h"
-#include "BitmapFormats.h"
+#include "NuxCore/Logger.h"
 #include "NuxCore/Math/MathFunctions.h"
 
+#include "BitmapFormats.h"
 #include "GdkGraphics.h"
 
 #if defined (NUX_OS_WINDOWS)
   #include "GdiImageLoader.h"
+  #include "DDS.h"
 #endif
 
+#include "ImageSurface.h"
+
 namespace nux
 {
+namespace
+{
+logging::Logger logger("nux.image");
+}
 
   extern PixelFormatInfo GPixelFormats[];
-  
+
   NBitmapData *LoadGdkPixbuf (GdkPixbuf *pixbuf)
   {
     unsigned int width = gdk_pixbuf_get_width (pixbuf);
     unsigned int height = gdk_pixbuf_get_height (pixbuf);
     unsigned int row_bytes = gdk_pixbuf_get_rowstride (pixbuf);
-    
+
     NTextureData *Texture = new NTextureData (BITFMT_R8G8B8A8, width, height, 1);
-    
+
     guchar *img = gdk_pixbuf_get_pixels (pixbuf);
-    
+
     for (unsigned int i = 0; i < width; i++)
       for (unsigned int j = 0; j < height; j++)
       {
@@ -57,29 +63,20 @@ namespace nux
           (* (pixels + 1) << 8)  |  // g
           * (pixels + 0);           // r
 
-        Texture->GetSurface (0).Write32b (i, j, value); // = vec4ub(img + ((h-j-1)*row_bytes + i * 4));
+        Texture->GetSurface (0).Write32b (i, j, value);
       }
-    
+
     return Texture;
   }
 
   NBitmapData *LoadImageFile (const TCHAR *filename)
   {
-//    if((filename == 0) || (filename == TEXT("")))
-//        return 0;
-
     if (!GFileManager.FileExist (filename) )
     {
       nuxAssertMsg (0, TEXT ("[LoadImageFile] Can't find file: %s"), filename);
       return 0;
     }
 
-//     GdkGraphics *gdkgraphics = new GdkGraphics ();
-//     if (gdkgraphics->LoadImage (filename))
-//     {
-//       return gdkgraphics->GetBitmap ();
-//     }
-
     NBitmapData *BitmapData = 0;
 
 #if defined(NUX_OS_WINDOWS)
@@ -89,26 +86,8 @@ namespace nux
     BitmapData = read_tga_file (filename);
     if (BitmapData) return BitmapData;
 
-//     BitmapData = read_bmp_file (filename);
-//     if (BitmapData) return BitmapData;
-// 
-//     BitmapData = read_png_rgba (filename);
-//     if (BitmapData) return BitmapData;
-// 
-//     BitmapData = read_png_rgb (filename);
-//     if (BitmapData) return BitmapData;
-// 
-//     BitmapData = Load_DDS_File (filename);
-//     if (BitmapData) return BitmapData;
-// 
-//     BitmapData = LoadRGBE (filename);
-//     if (BitmapData) return BitmapData;
-
-//     BitmapData = LoadAnimatedTextureFile (filename);
-//     if (BitmapData) return BitmapData;
-//     
-//     BitmapData = LoadITXFile (filename);
-//     if (BitmapData) return BitmapData;
+    BitmapData = LoadFileFormat_DDS(filename);
+    if (BitmapData) return BitmapData;
 
 #elif defined(NUX_OS_LINUX)
     GdkGraphics gdkgraphics;
@@ -118,7 +97,7 @@ namespace nux
 #endif
 
     // Unsupported format
-    nuxDebugMsg (TEXT ("[LoadImageFile] Unknown file format: %s."), filename);
+    LOG_DEBUG(logger) << "Unknown file format: " << filename;
     return 0;
   }
 
@@ -131,13 +110,14 @@ namespace nux
 #endif
   }
 
-  void MakeCheckBoardImage (ImageSurface &Image, t_s32 w, t_s32 h,
-                            Color color0,
-                            Color color1,
-                            t_s32 TileWidth,
-                            t_s32 TileHeight)
+  void MakeCheckBoardImage (ImageSurface &Image,
+                            int width, int height,
+                            Color const& dark,
+                            Color const& light,
+                            int TileWidth,
+                            int TileHeight)
   {
-    Image.Allocate (BITFMT_R8G8B8A8, w, h);
+    Image.Allocate (BITFMT_R8G8B8A8, width, height);
 
     if (TileWidth <= 0)
       TileWidth = 4;
@@ -145,23 +125,20 @@ namespace nux
     if (TileHeight <= 0)
       TileHeight = 4;
 
-    t_s32 i, j, c;
-
-    for (j = 0; j < h; j++)
+    for (int j = 0; j < height; ++j)
     {
-      for (i = 0; i < w; i++)
+      for (int i = 0; i < width; ++i)
       {
         /*every 8 bits, change color from black to white or vice versa */
-
-        c = ( ( ( i & TileWidth ) == 0 ) ^ ( ( j & TileHeight )  == 0 ) );
-        t_u32 a = ( ( ( i / TileWidth ) % 2) == 0 );
-        t_u32 b = ( ( ( j / TileHeight ) % 2) == 0 );
-        c = a ^ b;
-        t_u8 R = c ? color0.R() * 255 : color1.R() * 255;
-        t_u8 G = c ? color0.G() * 255 : color1.G() * 255;
-        t_u8 B = c ? color0.B() * 255 : color1.B() * 255;
-        t_u8 A = c ? color0.A() * 255 : color1.A() * 255;
-        Image.Write (i, j, R, G, B, A);
+        bool even_column = ((i / TileWidth) % 2) == 0;
+        bool even_row = ((j / TileHeight ) % 2) == 0;
+        bool is_dark = even_column ^ even_row;
+        Color const& c = is_dark ? dark : light;
+        Image.Write(i, j,
+                    c.red * 255,
+                    c.green * 255,
+                    c.blue * 255,
+                    c.alpha * 255);
       }
     }
 
@@ -177,11 +154,9 @@ namespace nux
 
   NBitmapData::~NBitmapData()
   {
-//     delete [] data_;
   }
 
 
-///////////////////////////////////////////////////////////////////////////////////////
   ImageSurface::ImageSurface()
     :   width_ (0)
     ,   height_ (0)
@@ -196,7 +171,7 @@ namespace nux
 
   ImageSurface::~ImageSurface()
   {
-    NUX_SAFE_DELETE (RawData_);
+    delete [] RawData_;
   }
 
   ImageSurface::ImageSurface (BitmapFormat format, t_u32 width, t_u32 height)
@@ -208,7 +183,6 @@ namespace nux
     ,   Alignment_ (4)
     ,   RawData_ (0)
   {
-    //MakeCheckBoardImage(*this, width, height, 0xFFBBBBBB, 0xFF444444, 4, 4);
     Allocate (format, width, height);
   }
 
@@ -221,8 +195,8 @@ namespace nux
     m_Pitch = surface.m_Pitch;
     Alignment_ = surface.Alignment_;
 
-    RawData_ = new t_u8[surface.GetSize() ];
-    Memcpy (RawData_, surface.RawData_, surface.GetSize() );
+    RawData_ = new t_u8[surface.GetSize()];
+    Memcpy(RawData_, surface.RawData_, surface.GetSize());
   }
 
   ImageSurface &ImageSurface::operator = (const ImageSurface &surface)
@@ -287,7 +261,7 @@ namespace nux
       return;
     }
 
-    NUX_SAFE_DELETE_ARRAY (RawData_);
+    delete [] RawData_;
 
     if ( (width == 0) || (height == 0) )
     {
@@ -315,8 +289,9 @@ namespace nux
          (format_ == BITFMT_DXT4) ||
          (format_ == BITFMT_DXT5) )
     {
-      // For DXT, width and height are rounded up to a multiple of 4 in order to create 4x4 blocks of pixels;
-      // And in this context, byte alignment is 1 ie. data is densely packed.
+      // For DXT, width and height are rounded up to a multiple of 4 in order
+      // to create 4x4 blocks of pixels; And in this context, byte alignment
+      // is 1 ie. data is densely packed.
       t_u32 block = GPixelFormats[format].BlockSizeX;
       t_u32 shift = Log2 (GPixelFormats[format].BlockSizeX);
       m_Pitch = Align ( (bpe_ * ( (width_ + (block - 1) ) >> shift) ), Alignment_);
@@ -339,11 +314,13 @@ namespace nux
     Clear();
   }
 
-// This computes the correct pitch of a line. For instance if the unpack alignment is 4, the pitch must have
-// a number of pixel multiple of 4.
-// See Avoiding 16 Common OpenGL Pitfalls http://www.opengl.org/resources/features/KilgardTechniques/oglpitfall/
+// This computes the correct pitch of a line. For instance if the unpack
+// alignment is 4, the pitch must have a number of pixel multiple of 4.  See
+// Avoiding 16 Common OpenGL Pitfalls
+// http://www.opengl.org/resources/features/KilgardTechniques/oglpitfall/
 // 7. Watch Your Pixel Store Alignment
-  t_s32 ImageSurface::GetLevelPitch (BitmapFormat format, t_s32 width, t_s32 height, t_s32 miplevel)
+  t_s32 ImageSurface::GetLevelPitch (BitmapFormat format, t_s32 width,
+                                     t_s32 height, t_s32 miplevel)
   {
     t_s32 levelwidth = ImageSurface::GetLevelDim (format, width, miplevel);
 
@@ -570,10 +547,8 @@ namespace nux
     if ( (width_ == 0) || (height_ == 0) )
       return;
 
-    for (i = 0; i < GetSize(); i++)
-    {
-      RawData_[i] = 0;
-    }
+    auto size = GetSize ();
+    memset(RawData_, 0, size);
   }
 
   void ImageSurface::FlipHorizontal()
@@ -622,12 +597,6 @@ namespace nux
     if ( (format_ == BITFMT_DXT1) || (format_ == BITFMT_DXT2) || (format_ == BITFMT_DXT3) || (format_ == BITFMT_DXT4) || (format_ == BITFMT_DXT5) )
     {
       FlipDXTVertical();
-//        t_u32 h = (height_ + 3) >> 2;
-//        flip_data =  new t_u8[m_Pitch*h];
-//        for(j = 0; j < h; j++)
-//        {
-//            Memcpy(flip_data + (h-j-1)*m_Pitch, RawData_+j*m_Pitch, m_Pitch);
-//        }
     }
     else
     {
@@ -917,89 +886,6 @@ namespace nux
     return Color (r / num_pixels, g / num_pixels, b / num_pixels, a / num_pixels);
   }
 
-  void ImageSurface::GaussianBlur (int radius)
-  {
-    NUX_RETURN_IF_FALSE (width_ != 0);
-    NUX_RETURN_IF_FALSE (height_ != 0);
-
-    ImageSurface temp (format_, width_, height_);
-
-    int gauss_w = 1 + 2 * 3;          // Level(+1)
-    int mask[7] = {1,6,15,20,15,6,1}; // Mask
-    int gauss_sum = 64;               // Sum
- 
-    t_u8 R, G, B, A;
-    double r = 0;
-    double g = 0;
-    double b = 0;
-    double a = 0;
-
-    // Horizontal blur
-    for (int j = 0; j < height_ - 1; j++)
-    {
-      for (int i = gauss_w - 1; i < width_ - 1; i++)
-      {
-        r = 0;
-        g = 0;
-        b = 0;
-        a = 0;
-
-        for (int k = 0; k < gauss_w; k++)
-        {
-          int color = Read(i - (gauss_w-1) + k, j);
-          R = NUX_RGBA_GET_RED (color);
-          G = NUX_RGBA_GET_GREEN (color);
-          B = NUX_RGBA_GET_BLUE (color);
-          A = NUX_RGBA_GET_ALPHA (color);
-
-          r += R * mask[k];
-          g += G * mask[k];
-          b += B * mask[k];
-          a += A * mask[k];
-        }
-
-        R = r / gauss_sum;
-        G = g / gauss_sum;
-        B = b / gauss_sum;
-        A = a / gauss_sum;
-
-        temp.Write32b(i, j, NUX_RGBA (R, G, B, A));
-      }
-    }
-
-    // Vertical blur
-    for (int j = gauss_w - 1; j < height_ - 1; j++)
-    {
-      for (int i = 0; i < width_ - 1; i++)
-      {
-        r = 0;
-        g = 0;
-        b = 0;
-        a = 0;
-
-        for (int k = 0; k < gauss_w; k++)
-        {
-          int color = Read(i, j - (gauss_w-1) + k);
-          R = NUX_RGBA_GET_RED (color);
-          G = NUX_RGBA_GET_GREEN (color);
-          B = NUX_RGBA_GET_BLUE (color);
-          A = NUX_RGBA_GET_ALPHA (color);
-
-          r += R * mask[k];
-          g += G * mask[k];
-          b += B * mask[k];
-          a += A * mask[k];
-        }
-
-        R = r / gauss_sum;
-        G = g / gauss_sum;
-        B = b / gauss_sum;
-        A = a / gauss_sum;
-
-        Write32b(i, j, NUX_RGBA (R, G, B, A));
-      }
-    }
-  }
 
   ///////////////////////////////////////////////////////////////////
   NTextureData::NTextureData (BitmapFormat f, t_s32 width, t_s32 height, t_s32 NumMipmap)
@@ -1287,7 +1173,7 @@ namespace nux
       m_MipSurfaceArray[mip].clear();
     }
 
-    NUX_SAFE_DELETE_ARRAY (m_MipSurfaceArray);
+    delete [] m_MipSurfaceArray;
   }
 
 //! Copy constructor
@@ -1442,7 +1328,7 @@ namespace nux
     }
 
     m_FrameTimeArray.clear();
-    NUX_SAFE_DELETE_ARRAY (m_MipSurfaceArray);
+    delete [] m_MipSurfaceArray;
   }
 
 //! Copy constructor
